@@ -241,28 +241,80 @@ void curses_table::update_data(vector<sinsp_sample_row>* data, bool force_select
 	}
 }
 
-void curses_table::print_wait()
+void curses_table::print_line_centered(string line, int32_t off)
 {
-	string wstr;
+	wattrset(m_tblwin, m_parent->m_colors[sinsp_cursesui::PROCESS]);
 
-	if(m_inspector->is_live())
+	if(line.size() < m_parent->m_screenw)
 	{
-		wstr = "Collecting Data";
+		mvwprintw(m_tblwin, 
+			m_parent->m_screenh / 2 + off,
+			m_parent->m_screenw / 2 - line.size() / 2, 
+			line.c_str());
 	}
 	else
 	{
-		if(m_parent->is_eof())
+		uint32_t spos = 0;
+
+		for(uint32_t j = 0;; j++)
 		{
-			wstr = "No Data For This View";
+			string ss = line.substr(spos, spos + m_parent->m_screenw);
+lo("2, %d %s\n", spos, ss.c_str());
+
+			mvwprintw(m_tblwin, 
+				m_parent->m_screenh / 2 + off + j,
+				0,
+				ss.c_str());
+
+			spos += m_parent->m_screenw;
+			if(spos >= line.size())
+			{
+				break;
+			}
 		}
 	}
+}
 
-	wattrset(m_tblwin, m_parent->m_colors[sinsp_cursesui::PROCESS]);
+void curses_table::print_wait()
+{
+	string wstr;
+	bool is_tracer_view = false;
 
-	mvwprintw(m_tblwin, 
-		m_parent->m_screenh / 2,
-		m_parent->m_screenw / 2 - wstr.size() / 2, 
-		wstr.c_str());	
+	sinsp_view_info* vinfo = m_parent->get_selected_view();
+	if(vinfo)
+	{
+		if(vinfo->m_id == "tracers" ||
+			vinfo->m_id == "tracer_ids")
+		{
+			is_tracer_view = true;
+		}
+	}
+	else
+	{
+		ASSERT(false);
+	}
+
+	if(is_tracer_view)
+	{
+		print_line_centered("No data for this view.");
+		print_line_centered("Note: in order to see any data here, you need to push tracers to sysdig from your app as described here: XXX.", 2);
+	}
+	else
+	{
+		if(m_inspector->is_live())
+		{
+			wstr = "Collecting Data";
+		}
+		else
+		{
+			if(m_parent->is_eof())
+			{
+				wstr = "No Data For This View";
+			}
+		}
+	
+		print_line_centered(wstr);
+	}
 }
 
 void curses_table::print_error(string wstr)
