@@ -32,6 +32,10 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #ifdef HAS_FILTERING
 #include "filter.h"
 #include "filterchecks.h"
+#include "value_parser.h"
+#ifndef _WIN32
+#include "arpa/inet.h"
+#endif
 
 #ifndef _GNU_SOURCE
 //
@@ -68,6 +72,9 @@ sinsp_filter_check_list::sinsp_filter_check_list()
 	add_filter_check(new sinsp_filter_check_utils());
 	add_filter_check(new sinsp_filter_check_fdlist());
 	add_filter_check(new sinsp_filter_check_k8s());
+	add_filter_check(new sinsp_filter_check_mesos());
+	add_filter_check(new sinsp_filter_check_tracer());
+	add_filter_check(new sinsp_filter_check_evtin_tracer());
 }
 
 sinsp_filter_check_list::~sinsp_filter_check_list()
@@ -150,7 +157,7 @@ sinsp_filter_check* sinsp_filter_check_list::new_filter_check_from_another(sinsp
 ///////////////////////////////////////////////////////////////////////////////
 // type-based comparison functions
 ///////////////////////////////////////////////////////////////////////////////
-bool flt_compare_uint64(ppm_cmp_operator op, uint64_t operand1, uint64_t operand2)
+bool flt_compare_uint64(cmpop op, uint64_t operand1, uint64_t operand2)
 {
 	switch(op)
 	{
@@ -172,7 +179,7 @@ bool flt_compare_uint64(ppm_cmp_operator op, uint64_t operand1, uint64_t operand
 	}
 }
 
-bool flt_compare_int64(ppm_cmp_operator op, int64_t operand1, int64_t operand2)
+bool flt_compare_int64(cmpop op, int64_t operand1, int64_t operand2)
 {
 	switch(op)
 	{
@@ -191,8 +198,8 @@ bool flt_compare_int64(ppm_cmp_operator op, int64_t operand1, int64_t operand2)
 	case CO_CONTAINS:
 		throw sinsp_exception("'contains' not supported for numeric filters");
 		return false;
-	case CO_IN:
-		throw sinsp_exception("'in' not supported for numeric filters");
+	case CO_ICONTAINS:
+		throw sinsp_exception("'icontains' not supported for numeric filters");
 		return false;
 	default:
 		throw sinsp_exception("'unknown' not supported for numeric filters");
@@ -200,7 +207,7 @@ bool flt_compare_int64(ppm_cmp_operator op, int64_t operand1, int64_t operand2)
 	}
 }
 
-bool flt_compare_string(ppm_cmp_operator op, char* operand1, char* operand2)
+bool flt_compare_string(cmpop op, char* operand1, char* operand2)
 {
 	switch(op)
 	{
@@ -210,8 +217,12 @@ bool flt_compare_string(ppm_cmp_operator op, char* operand1, char* operand2)
 		return (strcmp(operand1, operand2) != 0);
 	case CO_CONTAINS:
 		return (strstr(operand1, operand2) != NULL);
-	case CO_IN:
-		return (strstr(operand1, operand2) != NULL);
+    case CO_ICONTAINS:
+#ifdef _WIN32
+		return (_strnicmp(operand1, operand2, strlen(operand1)) != NULL);
+#else
+		return (strcasestr(operand1, operand2) != NULL);
+#endif
 	case CO_LT:
 		return (strcmp(operand1, operand2) < 0);
 	case CO_LE:
@@ -227,7 +238,7 @@ bool flt_compare_string(ppm_cmp_operator op, char* operand1, char* operand2)
 	}
 }
 
-bool flt_compare_buffer(ppm_cmp_operator op, char* operand1, char* operand2, uint32_t op1_len, uint32_t op2_len)
+bool flt_compare_buffer(cmpop op, char* operand1, char* operand2, uint32_t op1_len, uint32_t op2_len)
 {
 	switch(op)
 	{
@@ -237,6 +248,8 @@ bool flt_compare_buffer(ppm_cmp_operator op, char* operand1, char* operand2, uin
 		return op1_len != op2_len || (memcmp(operand1, operand2, op1_len) != 0);
 	case CO_CONTAINS:
 		return (memmem(operand1, op1_len, operand2, op2_len) != NULL);
+	case CO_ICONTAINS:
+		throw sinsp_exception("'icontains' not supported for buffer filters");
 	case CO_LT:
 		throw sinsp_exception("'<' not supported for buffer filters");
 	case CO_LE:
@@ -252,7 +265,7 @@ bool flt_compare_buffer(ppm_cmp_operator op, char* operand1, char* operand2, uin
 	}
 }
 
-bool flt_compare_double(ppm_cmp_operator op, double operand1, double operand2)
+bool flt_compare_double(cmpop op, double operand1, double operand2)
 {
 	switch(op)
 	{
@@ -274,7 +287,22 @@ bool flt_compare_double(ppm_cmp_operator op, double operand1, double operand2)
 	}
 }
 
-bool flt_compare(ppm_cmp_operator op, ppm_param_type type, void* operand1, void* operand2, uint32_t op1_len, uint32_t op2_len)
+bool flt_compare_ipv4net(cmpop op, uint64_t operand1, ipv4net* operand2)
+{
+	switch(op)
+	{
+	case CO_EQ:
+	{
+		return ((operand1 & operand2->m_netmask) == (operand2->m_ip & operand2->m_netmask));
+	}
+	case CO_NE:
+		return ((operand1 & operand2->m_netmask) != (operand2->m_ip && operand2->m_netmask));
+	default:
+		throw sinsp_exception("comparison operator not supported for ipv4 networks");
+	}
+}
+
+bool flt_compare(cmpop op, ppm_param_type type, void* operand1, void* operand2, uint32_t op1_len, uint32_t op2_len)
 {
 	//
 	// sinsp_filter_check_*::compare
@@ -312,6 +340,8 @@ bool flt_compare(ppm_cmp_operator op, ppm_param_type type, void* operand1, void*
 	case PT_BOOL:
 	case PT_IPV4ADDR:
 		return flt_compare_uint64(op, (uint64_t)*(uint32_t*)operand1, (uint64_t)*(uint32_t*)operand2);
+	case PT_IPV4NET:
+		return flt_compare_ipv4net(op, (uint64_t)*(uint32_t*)operand1, (ipv4net*)operand2);
 	case PT_UINT64:
 	case PT_RELTIME:
 	case PT_ABSTIME:
@@ -333,7 +363,7 @@ bool flt_compare(ppm_cmp_operator op, ppm_param_type type, void* operand1, void*
 	}
 }
 
-bool flt_compare_avg(ppm_cmp_operator op, 
+bool flt_compare_avg(cmpop op, 
 					 ppm_param_type type, 
 					 void* operand1, 
 					 void* operand2, 
@@ -438,8 +468,7 @@ bool flt_compare_avg(ppm_cmp_operator op,
 ///////////////////////////////////////////////////////////////////////////////
 // sinsp_filter_check implementation
 ///////////////////////////////////////////////////////////////////////////////
-sinsp_filter_check::sinsp_filter_check() :
-	m_val_storage(256)
+sinsp_filter_check::sinsp_filter_check()
 {
 	m_boolop = BO_NONE;
 	m_cmpop = CO_NONE;
@@ -450,6 +479,7 @@ sinsp_filter_check::sinsp_filter_check() :
 	m_val_storage_len = 0;
 	m_aggregation = A_NONE;
 	m_merge_aggregation = A_NONE;
+	m_val_storages = vector<vector<uint8_t>> (1, vector<uint8_t>(256));
 }
 
 void sinsp_filter_check::set_inspector(sinsp* inspector)
@@ -806,14 +836,14 @@ char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_fi
 			{
 				ASSERT(len < 1024 * 1024);
 
-				if(len >= m_val_storage.size())
+				if(len >= filter_value().size())
 				{
-					m_val_storage.resize(len + 1);
+					filter_value().resize(len + 1);
 				}
 
-				memcpy(&m_val_storage[0], rawval, len);
-				m_val_storage[len] = 0;
-				return (char*)&m_val_storage[0];
+				memcpy(filter_value_p(), rawval, len);
+				filter_value_p()[len] = 0;
+				return (char*)filter_value_p();
 			}
 		case PT_SOCKADDR:
 			ASSERT(false);
@@ -850,117 +880,6 @@ char* sinsp_filter_check::rawval_to_string(uint8_t* rawval, const filtercheck_fi
 	}
 }
 
-void sinsp_filter_check::string_to_rawval(const char* str, uint32_t len, ppm_param_type ptype)
-{
-	switch(ptype)
-	{
-		case PT_INT8:
-			*(int8_t*)(&m_val_storage[0]) = sinsp_numparser::parsed8(str);
-			break;
-		case PT_INT16:
-			*(int16_t*)(&m_val_storage[0]) = sinsp_numparser::parsed16(str);
-			break;
-		case PT_INT32:
-			*(int32_t*)(&m_val_storage[0]) = sinsp_numparser::parsed32(str);
-			break;
-		case PT_INT64:
-		case PT_FD:
-		case PT_ERRNO:
-			*(int64_t*)(&m_val_storage[0]) = sinsp_numparser::parsed64(str);
-			break;
-		case PT_L4PROTO: // This can be resolved in the future
-		case PT_FLAGS8:
-		case PT_UINT8:
-			*(uint8_t*)(&m_val_storage[0]) = sinsp_numparser::parseu8(str);
-			break;
-		case PT_PORT:
-		{
-			string in(str);
-
-			if(in.empty())
-			{
-				*(uint16_t*)(&m_val_storage[0]) = 0;
-			}
-			else
-			{
-				// if the string is made only of numbers
-				if(strspn(in.c_str(), "0123456789") == in.size())
-				{
-					*(uint16_t*)(&m_val_storage[0]) = stoi(in);
-				}
-				else
-				{
-					*(uint16_t*)(&m_val_storage[0]) = ntohs(getservbyname(in.c_str(), NULL)->s_port);
-				}
-			}
-
-			break;
-		}
-		case PT_FLAGS16:
-		case PT_UINT16:
-			*(uint16_t*)(&m_val_storage[0]) = sinsp_numparser::parseu16(str);
-			break;
-		case PT_FLAGS32:
-		case PT_UINT32:
-			*(uint32_t*)(&m_val_storage[0]) = sinsp_numparser::parseu32(str);
-			break;
-		case PT_UINT64:
-			*(uint64_t*)(&m_val_storage[0]) = sinsp_numparser::parseu64(str);
-			break;
-		case PT_RELTIME:
-		case PT_ABSTIME:
-			*(uint64_t*)(&m_val_storage[0]) = sinsp_numparser::parseu64(str);
-			break;
-		case PT_CHARBUF:
-		case PT_SOCKADDR:
-		case PT_SOCKFAMILY:
-			{
-				len = (uint32_t)strlen(str);
-				if(len >= m_val_storage.size())
-				{
-					throw sinsp_exception("filter parameter too long:" + string(str));
-				}
-
-				memcpy((&m_val_storage[0]), str, len);
-				m_val_storage[len] = 0;
-			}
-			break;
-		case PT_BOOL:
-			if(string(str) == "true")
-			{
-				*(uint32_t*)(&m_val_storage[0]) = 1;
-			}
-			else if(string(str) == "false")
-			{
-				*(uint32_t*)(&m_val_storage[0]) = 0;
-			}
-			else
-			{
-				throw sinsp_exception("filter error: unrecognized boolean value " + string(str));
-			}
-
-			break;
-		case PT_IPV4ADDR:
-			if(inet_pton(AF_INET, str, (&m_val_storage[0])) != 1)
-			{
-				throw sinsp_exception("unrecognized IP address " + string(str));
-			}
-			break;
-		case PT_BYTEBUF:
-			if(len >= m_val_storage.size())
-			{
-				throw sinsp_exception("filter parameter too long:" + string(str));
-			}
-
-			memcpy((&m_val_storage[0]), str, len);
-			m_val_storage_len = len;
-			break;
-		default:
-			ASSERT(false);
-			throw sinsp_exception("wrong event type " + to_string((long long) m_field->m_type));
-	}
-}
-
 char* sinsp_filter_check::tostring(sinsp_evt* evt)
 {
 	uint32_t len;
@@ -1023,9 +942,47 @@ int32_t sinsp_filter_check::parse_field_name(const char* str, bool alloc_state)
 	return max_fldlen;
 }
 
-void sinsp_filter_check::parse_filter_value(const char* str, uint32_t len)
+void sinsp_filter_check::set_check_id(int32_t id)
 {
-	string_to_rawval(str, len, m_field->m_type);
+	m_check_id = id;
+}
+
+int32_t sinsp_filter_check::get_check_id()
+{
+	return m_check_id;
+}
+
+
+void sinsp_filter_check::add_filter_value(const char* str, uint32_t len, uint16_t i)
+{
+
+	if (i >= m_val_storages.size())
+	{
+		m_val_storages.push_back(vector<uint8_t>(256));
+	}
+
+	parse_filter_value(str, len, filter_value_p(i), filter_value(i).size());
+}
+
+
+void sinsp_filter_check::parse_filter_value(const char* str, uint32_t len, uint8_t *storage, uint32_t storage_len)
+{
+	// byte buffer, no parsing needed
+	if (m_field->m_type == PT_BYTEBUF)
+	{
+		if(len >= storage_len)
+		{
+			throw sinsp_exception("filter parameter too long:" + string(str));
+		}
+		memcpy(storage, str, len);
+		m_val_storage_len = len;
+		return;
+	}
+	else
+	{
+		sinsp_filter_value_parser::string_to_rawval(str, len, storage, storage_len, m_field->m_type);
+	}
+	validate_filter_value(str, len);
 }
 
 const filtercheck_field_info* sinsp_filter_check::get_field_info()
@@ -1033,10 +990,42 @@ const filtercheck_field_info* sinsp_filter_check::get_field_info()
 	return &m_info.m_fields[m_field_id];
 }
 
+bool sinsp_filter_check::flt_compare(cmpop op, ppm_param_type type, void* operand1, uint32_t op1_len, uint32_t op2_len)
+{
+	if (op == CO_IN)
+	{
+		if (op1_len)
+		{
+			throw sinsp_exception("filter error: cannot use 'in' operator with param type "+ to_string(type));
+		}
+		for (uint16_t i=0; i < m_val_storages.size(); i++)
+		{
+			if (::flt_compare(CO_EQ,
+					  type,
+					  operand1,
+					  filter_value_p(i)))
+			{
+				return true;
+			}
+		}
+		return false;
+	}
+	else
+	{
+		return (::flt_compare(op,
+				      type,
+				      operand1,
+				      filter_value_p(),
+				      op1_len,
+				      op2_len)
+			);
+	}
+}
+
 bool sinsp_filter_check::compare(sinsp_evt *evt)
 {
-	uint32_t len;
-	uint8_t* extracted_val = extract(evt, &len);
+	uint32_t evt_val_len=0;
+	uint8_t* extracted_val = extract(evt, &evt_val_len);
 
 	if(extracted_val == NULL)
 	{
@@ -1044,11 +1033,10 @@ bool sinsp_filter_check::compare(sinsp_evt *evt)
 	}
 
 	return flt_compare(m_cmpop,
-		m_info.m_fields[m_field_id].m_type,
-		extracted_val,
-		&m_val_storage[0],
-		len,
-		m_val_storage_len);
+			   m_info.m_fields[m_field_id].m_type,
+			   extracted_val,
+			   evt_val_len,
+			   m_val_storage_len);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1069,6 +1057,12 @@ sinsp_filter_expression::~sinsp_filter_expression()
 	}
 }
 
+// Only filter checks get IDs
+int32_t sinsp_filter_expression::get_check_id()
+{
+	return 0;
+}
+
 sinsp_filter_check* sinsp_filter_expression::allocate_new()
 {
 	ASSERT(false);
@@ -1089,10 +1083,11 @@ bool sinsp_filter_expression::compare(sinsp_evt *evt)
 	uint32_t j;
 	uint32_t size = (uint32_t)m_checks.size();
 	bool res = true;
+	sinsp_filter_check* chk = NULL;
 
 	for(j = 0; j < size; j++)
 	{
-		sinsp_filter_check* chk = m_checks[j];
+		chk = m_checks[j];
 		ASSERT(chk != NULL);
 
 		if(j == 0)
@@ -1101,6 +1096,9 @@ bool sinsp_filter_expression::compare(sinsp_evt *evt)
 			{
 			case BO_NONE:
 				res = chk->compare(evt);
+				if (res) {
+					evt->set_check_id(chk->get_check_id());
+				}
 				break;
 			case BO_NOT:
 				res = !chk->compare(evt);
@@ -1115,16 +1113,44 @@ bool sinsp_filter_expression::compare(sinsp_evt *evt)
 			switch(chk->m_boolop)
 			{
 			case BO_OR:
-				res = res || chk->compare(evt);
+				if(res)
+				{
+					goto done;
+				}
+				res = chk->compare(evt);
+				if (res) {
+					evt->set_check_id(chk->get_check_id());
+				}
 				break;
 			case BO_AND:
-				res = res && chk->compare(evt);
+				if(!res)
+				{
+					goto done;
+				}
+				res = chk->compare(evt);
+				if (res) {
+					evt->set_check_id(chk->get_check_id());
+				}
 				break;
 			case BO_ORNOT:
-				res = res || !chk->compare(evt);
+				if(res)
+				{
+					goto done;
+				}
+				res = !chk->compare(evt);
+				if (res) {
+					evt->set_check_id(chk->get_check_id());
+				}
 				break;
 			case BO_ANDNOT:
-				res = res && !chk->compare(evt);
+				if(!res)
+				{
+					goto done;
+				}
+				res = !chk->compare(evt);
+				if (res) {
+					evt->set_check_id(chk->get_check_id());
+				}
 				break;
 			default:
 				ASSERT(false);
@@ -1132,6 +1158,7 @@ bool sinsp_filter_expression::compare(sinsp_evt *evt)
 			}
 		}
 	}
+ done:
 
 	return res;
 }
@@ -1139,32 +1166,12 @@ bool sinsp_filter_expression::compare(sinsp_evt *evt)
 ///////////////////////////////////////////////////////////////////////////////
 // sinsp_filter implementation
 ///////////////////////////////////////////////////////////////////////////////
-sinsp_filter::sinsp_filter(sinsp* inspector, const string& fltstr, bool ttable_only)
+sinsp_filter::sinsp_filter(sinsp* inspector)
 {
 	m_inspector = inspector;
-	m_ttable_only = ttable_only;
-	m_scanpos = -1;
-	m_scansize = 0;
-	m_state = ST_NEED_EXPRESSION;
 	m_filter = new sinsp_filter_expression();
 	m_curexpr = m_filter;
-	m_last_boolop = BO_NONE;
-	m_nest_level = 0;
 
-	try
-	{
-		compile(fltstr);
-	}
-	catch(sinsp_exception& e)
-	{
-		delete m_filter;
-		throw e;
-	}
-	catch(...)
-	{
-		delete m_filter;
-		throw sinsp_exception("error parsing the filter string");
-	}
 }
 
 sinsp_filter::~sinsp_filter()
@@ -1175,7 +1182,57 @@ sinsp_filter::~sinsp_filter()
 	}
 }
 
-bool sinsp_filter::isblank(char c)
+void sinsp_filter::push_expression(boolop op)
+{
+	sinsp_filter_expression* newexpr = new sinsp_filter_expression();
+	newexpr->m_boolop = op;
+	newexpr->m_parent = m_curexpr;
+
+	add_check((sinsp_filter_check*)newexpr);
+	m_curexpr = newexpr;
+}
+
+void sinsp_filter::pop_expression()
+{
+	ASSERT(m_curexpr->m_parent != NULL);
+
+	m_curexpr = m_curexpr->m_parent;
+}
+
+bool sinsp_filter::run(sinsp_evt *evt)
+{
+	//	printf("m_filter: %p", (void*) m_filter);
+	//	ASSERT(m_filter != NULL);
+	return m_filter->compare(evt);
+}
+
+void sinsp_filter::add_check(sinsp_filter_check* chk)
+{
+	m_curexpr->add_check(chk);
+}
+
+///////////////////////////////////////////////////////////////////////////////
+// sinsp_filter_compiler implementation
+///////////////////////////////////////////////////////////////////////////////
+sinsp_filter_compiler::sinsp_filter_compiler(sinsp* inspector, const string& fltstr, bool ttable_only)
+{
+	m_inspector = inspector;
+	m_ttable_only = ttable_only;
+	m_scanpos = -1;
+	m_scansize = 0;
+	m_state = ST_NEED_EXPRESSION;
+	m_filter = new sinsp_filter(m_inspector);
+	m_last_boolop = BO_NONE;
+	m_nest_level = 0;
+	m_fltstr = fltstr;
+
+}
+
+sinsp_filter_compiler::~sinsp_filter_compiler()
+{
+}
+
+bool sinsp_filter_compiler::isblank(char c)
 {
 	if(c == ' ' || c == '\t' || c == '\n' || c == '\r')
 	{
@@ -1187,7 +1244,7 @@ bool sinsp_filter::isblank(char c)
 	}
 }
 
-bool sinsp_filter::is_special_char(char c)
+bool sinsp_filter_compiler::is_special_char(char c)
 {
 	if(c == '(' || c == ')' || c == '!' || c == '=' || c == '<' || c == '>')
 	{
@@ -1197,7 +1254,7 @@ bool sinsp_filter::is_special_char(char c)
 	return false;
 }
 
-bool sinsp_filter::is_bracket(char c)
+bool sinsp_filter_compiler::is_bracket(char c)
 {
 	if(c == '(' || c == ')')
 	{
@@ -1207,7 +1264,7 @@ bool sinsp_filter::is_bracket(char c)
 	return false;
 }
 
-char sinsp_filter::next()
+char sinsp_filter_compiler::next()
 {
 	while(true)
 	{
@@ -1225,7 +1282,7 @@ char sinsp_filter::next()
 	}
 }
 
-vector<char> sinsp_filter::next_operand(bool expecting_first_operand, bool in_clause)
+vector<char> sinsp_filter_compiler::next_operand(bool expecting_first_operand, bool in_clause)
 {
 	vector<char> res;
 	bool is_quoted = false;
@@ -1379,7 +1436,7 @@ vector<char> sinsp_filter::next_operand(bool expecting_first_operand, bool in_cl
 	return res;
 }
 
-bool sinsp_filter::compare_no_consume(const string& str)
+bool sinsp_filter_compiler::compare_no_consume(const string& str)
 {
 	//
 	// If the rest of the filter cannot contain the operand we may return
@@ -1402,7 +1459,7 @@ bool sinsp_filter::compare_no_consume(const string& str)
 	}
 }
 
-ppm_cmp_operator sinsp_filter::next_comparison_operator()
+cmpop sinsp_filter_compiler::next_comparison_operator()
 {
 	int32_t start;
 
@@ -1454,6 +1511,11 @@ ppm_cmp_operator sinsp_filter::next_comparison_operator()
 		m_scanpos += 8;
 		return CO_CONTAINS;
 	}
+	else if(compare_no_consume("icontains"))
+	{
+		m_scanpos += 9;
+		return CO_ICONTAINS;
+	}
 	else if(compare_no_consume("in"))
 	{
 		m_scanpos += 2;
@@ -1470,12 +1532,13 @@ ppm_cmp_operator sinsp_filter::next_comparison_operator()
 	}
 }
 
-void sinsp_filter::parse_check(sinsp_filter_expression* parent_expr, boolop op)
+void sinsp_filter_compiler::parse_check()
 {
 	uint32_t startpos = m_scanpos;
 	vector<char> operand1 = next_operand(true, false);
 	string str_operand1 = string((char *)&operand1[0]);
 	sinsp_filter_check* chk = g_filterlist.new_filter_check_from_fldname(str_operand1, m_inspector, true);
+	boolop op = m_last_boolop;
 
 	if(chk == NULL)
 	{
@@ -1491,7 +1554,7 @@ void sinsp_filter::parse_check(sinsp_filter_expression* parent_expr, boolop op)
 		}
 	}
 
-	ppm_cmp_operator co = next_comparison_operator();
+	cmpop co = next_comparison_operator();
 
 	chk->m_boolop = op;
 	chk->m_cmpop = co;
@@ -1507,7 +1570,9 @@ void sinsp_filter::parse_check(sinsp_filter_expression* parent_expr, boolop op)
 		// Separate the 'or's from the
 		// rest of the conditions
 		//
-		push_expression(op);
+		m_filter->push_expression(op);
+		m_last_boolop = BO_NONE;
+		m_nest_level++;
 
 		//
 		// Skip spaces
@@ -1547,15 +1612,9 @@ void sinsp_filter::parse_check(sinsp_filter_expression* parent_expr, boolop op)
 			sinsp_filter_check* newchk = g_filterlist.new_filter_check_from_another(chk);
 			newchk->m_boolop = op;
 			newchk->m_cmpop = CO_EQ;
-			newchk->parse_filter_value((char *)&operand2[0], (uint32_t)operand2.size() - 1);
+			newchk->add_filter_value((char *)&operand2[0], (uint32_t)operand2.size() - 1);
 
-			//
-			// We pushed another expression before
-			// so 'parent_expr' still referers to
-			// the old one, this is the new nested
-			// level for the 'or' sequence
-			//
-			m_curexpr->add_check(newchk);
+			m_filter->add_check(newchk);
 
 			next();
 
@@ -1581,7 +1640,8 @@ void sinsp_filter::parse_check(sinsp_filter_expression* parent_expr, boolop op)
 		//
 		// Come back to the rest of the filter
 		//
-		pop_expression();
+		m_filter->pop_expression();
+		m_nest_level--;
 	}
 	else
 	{
@@ -1600,36 +1660,33 @@ void sinsp_filter::parse_check(sinsp_filter_expression* parent_expr, boolop op)
 		else
 		{
 			vector<char> operand2 = next_operand(false, false);
-			chk->parse_filter_value((char *)&operand2[0], (uint32_t)operand2.size() - 1);
+			chk->add_filter_value((char *)&operand2[0], (uint32_t)operand2.size() - 1);
 		}
 
-		parent_expr->add_check(chk);
+		m_filter->add_check(chk);
 	}
 }
 
-void sinsp_filter::push_expression(boolop op)
+sinsp_filter* sinsp_filter_compiler::compile()
 {
-	sinsp_filter_expression* newexpr = new sinsp_filter_expression();
-	newexpr->m_boolop = op;
-	newexpr->m_parent = m_curexpr;
-	m_last_boolop = BO_NONE;
-
-	m_curexpr->m_checks.push_back((sinsp_filter_check*)newexpr);
-	m_curexpr = newexpr;
-	m_nest_level++;
+	try
+	{
+		return compile_();
+	}
+	catch(sinsp_exception& e)
+	{
+		delete m_filter;
+		throw e;
+	}
+	catch(...)
+	{
+		delete m_filter;
+		throw sinsp_exception("error parsing the filter string");
+	}
 }
 
-void sinsp_filter::pop_expression()
+sinsp_filter* sinsp_filter_compiler::compile_()
 {
-	ASSERT(m_curexpr->m_parent != NULL);
-
-	m_curexpr = m_curexpr->m_parent;
-	m_nest_level--;
-}
-
-void sinsp_filter::compile(const string& fltstr)
-{
-	m_fltstr = fltstr;
 	m_scansize = (uint32_t)m_fltstr.size();
 
 	while(true)
@@ -1655,7 +1712,7 @@ void sinsp_filter::compile(const string& fltstr)
 			//
 			// Good filter
 			//
-			return;
+			return m_filter;
 
 			break;
 		case '(':
@@ -1664,47 +1721,67 @@ void sinsp_filter::compile(const string& fltstr)
 				throw sinsp_exception("unexpected '(' after " + m_fltstr.substr(0, m_scanpos));
 			}
 
-			push_expression(m_last_boolop);
+			m_filter->push_expression(m_last_boolop);
+			m_last_boolop = BO_NONE;
+			m_nest_level++;
 
 			break;
 		case ')':
-			pop_expression();
+			m_filter->pop_expression();
+			m_nest_level--;
 			break;
 		case 'o':
-			if(next() == 'r')
+			if(m_scanpos != 0 && m_state != ST_NEED_EXPRESSION)
 			{
-				m_last_boolop = BO_OR;
+				if(next() == 'r')
+				{
+					m_last_boolop = BO_OR;
+				}
+				else
+				{
+					throw sinsp_exception("syntax error in filter at position " + to_string((long long)m_scanpos));
+				}
+
+				if(m_state != ST_EXPRESSION_DONE)
+				{
+					throw sinsp_exception("unexpected 'or' after " + m_fltstr.substr(0, m_scanpos));
+				}
+
+				m_state = ST_NEED_EXPRESSION;
 			}
 			else
 			{
-				throw sinsp_exception("syntax error in filter at position " + to_string((long long) m_scanpos));
+				parse_check();
+				m_state = ST_EXPRESSION_DONE;
 			}
 
-			if(m_state != ST_EXPRESSION_DONE)
-			{
-				throw sinsp_exception("unexpected 'or' after " + m_fltstr.substr(0, m_scanpos));
-			}
-
-			m_state = ST_NEED_EXPRESSION;
-
 			break;
 		case 'a':
-			if(next() == 'n' && next() == 'd')
+			if(m_scanpos != 0 && m_state != ST_NEED_EXPRESSION)
 			{
-				m_last_boolop = BO_AND;
+
+				if(next() == 'n' && next() == 'd')
+				{
+					m_last_boolop = BO_AND;
+				}
+				else
+				{
+					throw sinsp_exception("syntax error in filter at position " + to_string((long long)m_scanpos));
+				}
+
+				if(m_state != ST_EXPRESSION_DONE)
+				{
+					throw sinsp_exception("unexpected 'and' after " + m_fltstr.substr(0, m_scanpos));
+				}
+
+				m_state = ST_NEED_EXPRESSION;
 			}
 			else
 			{
-				throw sinsp_exception("syntax error in filter at position " + to_string((long long) m_scanpos));
+				parse_check();
+				m_state = ST_EXPRESSION_DONE;
 			}
 
-			if(m_state != ST_EXPRESSION_DONE)
-			{
-				throw sinsp_exception("unexpected 'and' after " + m_fltstr.substr(0, m_scanpos));
-			}
-
-			m_state = ST_NEED_EXPRESSION;
-
 			break;
 		case 'n':
 			if(next() == 'o' && next() == 't')
@@ -1727,7 +1804,7 @@ void sinsp_filter::compile(const string& fltstr)
 		default:
 			if(m_state == ST_NEED_EXPRESSION)
 			{
-				parse_check(m_curexpr, m_last_boolop);
+				parse_check();
 
 				m_state = ST_EXPRESSION_DONE;
 			}
@@ -1740,11 +1817,7 @@ void sinsp_filter::compile(const string& fltstr)
 	}
 
 	vector<string> components = sinsp_split(m_fltstr, ' ');
-}
-
-bool sinsp_filter::run(sinsp_evt *evt)
-{
-	return m_filter->compare(evt);
+	return m_filter;
 }
 
 #endif // HAS_FILTERING
