@@ -48,33 +48,30 @@ void k8s_collector::add(k8s_http* handler)
 	m_subscription_count = m_sockets.size();
 }
 
-void k8s_collector::remove(socket_map_t::iterator it)
+void k8s_collector::remove(socket_map_t::iterator& it)
 {
 	if(it != m_sockets.end())
 	{
-		m_sockets.erase(it);
+		m_sockets.erase(it++);
 	}
 	m_nfds = 0;
-	for (auto& sock : m_sockets)
+	for(const auto& sock : m_sockets)
 	{
 		if(sock.first > m_nfds)
 		{
 			m_nfds = sock.first;
 		}
 	}
-
 	m_subscription_count = m_sockets.size();
 }
 
 void k8s_collector::remove_all()
 {
 	K8S_LOCK_GUARD_MUTEX;
-
 	clear();
-	for (socket_map_t::iterator it = m_sockets.begin(); it != m_sockets.end();)
-	{
-		remove(it++);
-	}
+	m_sockets.clear();
+	m_nfds = 0;
+	m_subscription_count = 0;
 }
 
 bool k8s_collector::is_active() const
@@ -108,36 +105,45 @@ void k8s_collector::get_data()
 					if(res < 0) // error
 					{
 						std::string err = strerror(errno);
-						g_logger.log(err, sinsp_logger::SEV_CRITICAL);
+						g_logger.log(err, sinsp_logger::SEV_ERROR);
 						remove_all();
 					}
 					else // data or idle
 					{
-						for (auto& sock : m_sockets)
+						for(socket_map_t::iterator it = m_sockets.begin(); it != m_sockets.end();)
 						{
-							if(FD_ISSET(sock.first, &m_infd))
+							if(FD_ISSET(it->first, &m_infd))
 							{
-								if(!sock.second->on_data())
+								if(it->second && !it->second->on_data())
 								{
-									remove(m_sockets.find(sock.first));
+									if(errno != EAGAIN)
+									{
+										remove(it);
+										continue;
+									}
 								}
 							}
 							else
 							{
-								FD_SET(sock.first, &m_infd);
+								FD_SET(it->first, &m_infd);
 							}
 
-							if(FD_ISSET(sock.first, &m_errfd))
+							if(FD_ISSET(it->first, &m_errfd))
 							{
-								std::string err = strerror(errno);
-								g_logger.log(err, sinsp_logger::SEV_CRITICAL);
-								sock.second->on_error(err, true);
-								remove(m_sockets.find(sock.first));
+								if(errno != EAGAIN)
+								{
+									std::string err = strerror(errno);
+									g_logger.log(err, sinsp_logger::SEV_ERROR);
+									it->second->on_error(err, true);
+									remove(it);
+									continue;
+								}
 							}
 							else
 							{
-								FD_SET(sock.first, &m_errfd);
+								FD_SET(it->first, &m_errfd);
 							}
+							++it;
 						}
 					}
 				}
