@@ -170,7 +170,6 @@ void k8s_dispatcher::handle_node(const Json::Value& root, const msg_data& data)
 
 	if(data.m_reason == COMPONENT_ADDED)
 	{
-		std::vector<std::string> addresses = k8s_component::extract_nodes_addresses(root["status"]);
 		if(m_state.has(m_state.get_nodes(), data.m_uid))
 		{
 			std::ostringstream os;
@@ -178,13 +177,18 @@ void k8s_dispatcher::handle_node(const Json::Value& root, const msg_data& data)
 			g_logger.log(os.str(), sinsp_logger::SEV_INFO);
 		}
 		k8s_node_t& node = m_state.get_component<k8s_nodes, k8s_node_t>(m_state.get_nodes(), data.m_name, data.m_uid);
-		if(addresses.size() > 0)
-		{
-			node.set_host_ips(std::move(addresses));
-		}
-		Json::Value object = root["object"];
+		const Json::Value& object = root["object"];
 		if(!object.isNull())
 		{
+			const Json::Value& status = object["status"];
+			if(!status.isNull())
+			{
+				k8s_node_t::host_ip_list addresses = k8s_node_t::extract_addresses(status);
+				if(addresses.size() > 0)
+				{
+					node.set_host_ips(std::move(addresses));
+				}
+			}
 			Json::Value metadata = object["metadata"];
 			if(!metadata.isNull())
 			{
@@ -198,7 +202,6 @@ void k8s_dispatcher::handle_node(const Json::Value& root, const msg_data& data)
 	}
 	else if(data.m_reason == COMPONENT_MODIFIED)
 	{
-		std::vector<std::string> addresses = k8s_component::extract_nodes_addresses(root["status"]);
 		if(!m_state.has(m_state.get_nodes(), data.m_uid))
 		{
 			std::ostringstream os;
@@ -207,14 +210,19 @@ void k8s_dispatcher::handle_node(const Json::Value& root, const msg_data& data)
 			return;
 		}
 		k8s_node_t& node = m_state.get_component<k8s_nodes, k8s_node_t>(m_state.get_nodes(), data.m_name, data.m_uid);
-		if(addresses.size() > 0)
-		{
-			node.add_host_ips(std::move(addresses));
-		}
 		Json::Value object = root["object"];
 		if(!object.isNull())
 		{
-			Json::Value metadata = object["metadata"];
+			const Json::Value& status = object["status"];
+			if(!status.isNull())
+			{
+				k8s_node_t::host_ip_list addresses = k8s_node_t::extract_addresses(status);
+				if(addresses.size() > 0)
+				{
+					node.set_host_ips(std::move(addresses));
+				}
+			}
+			const Json::Value& metadata = object["metadata"];
 			if(!metadata.isNull())
 			{
 				k8s_pair_list entries = k8s_component::extract_object(metadata, "labels");
@@ -232,10 +240,14 @@ void k8s_dispatcher::handle_node(const Json::Value& root, const msg_data& data)
 			g_logger.log(std::string("NODE not found: ") + data.m_name, sinsp_logger::SEV_ERROR);
 		}
 	}
-	else // COMPONENT_ERROR
+	else if(data.m_reason == COMPONENT_ERROR)
 	{
 		log_error(root, "NODE");
 	}
+	else
+	{
+		g_logger.log(std::string("Unsupported K8S NODE event reason: ") + std::to_string(data.m_reason), sinsp_logger::SEV_ERROR);
+	}
 }
 
 void k8s_dispatcher::handle_namespace(const Json::Value& root, const msg_data& data)
@@ -251,10 +263,10 @@ void k8s_dispatcher::handle_namespace(const Json::Value& root, const msg_data& d
 			g_logger.log(os.str(), sinsp_logger::SEV_INFO);
 		}
 		k8s_ns_t& ns = m_state.get_component<k8s_namespaces, k8s_ns_t>(m_state.get_namespaces(), data.m_name, data.m_uid);
-		Json::Value object = root["object"];
+		const Json::Value& object = root["object"];
 		if(!object.isNull())
 		{
-			Json::Value metadata = object["metadata"];
+			const Json::Value& metadata = object["metadata"];
 			if(!metadata.isNull())
 			{
 				k8s_pair_list entries = k8s_component::extract_object(metadata, "labels");
@@ -270,15 +282,15 @@ void k8s_dispatcher::handle_namespace(const Json::Value& root, const msg_data& d
 		if(!m_state.has(m_state.get_namespaces(), data.m_uid))
 		{
 			std::ostringstream os;
-			os << "MODIFIED message received for non-existing node [" << data.m_uid << "], giving up.";
+			os << "MODIFIED message received for non-existing namespace [" << data.m_uid << "], giving up.";
 			g_logger.log(os.str(), sinsp_logger::SEV_ERROR);
 			return;
 		}
 		k8s_ns_t& ns = m_state.get_component<k8s_namespaces, k8s_ns_t>(m_state.get_namespaces(), data.m_name, data.m_uid);
-		Json::Value object = root["object"];
+		const Json::Value& object = root["object"];
 		if(!object.isNull())
 		{
-			Json::Value metadata = object["metadata"];
+			const Json::Value& metadata = object["metadata"];
 			if(!metadata.isNull())
 			{
 				k8s_pair_list entries = k8s_component::extract_object(metadata, "labels");
@@ -296,10 +308,14 @@ void k8s_dispatcher::handle_namespace(const Json::Value& root, const msg_data& d
 			g_logger.log(std::string("NAMESPACE not found: ") + data.m_name, sinsp_logger::SEV_ERROR);
 		}
 	}
-	else // COMPONENT_ERROR
+	else if(data.m_reason == COMPONENT_ERROR)
 	{
 		log_error(root, "NAMESPACE");
 	}
+	else
+	{
+		g_logger.log(std::string("Unsupported K8S NAMESPACE event reason: ") + std::to_string(data.m_reason), sinsp_logger::SEV_ERROR);
+	}
 }
 
 void k8s_dispatcher::handle_pod(const Json::Value& root, const msg_data& data)
@@ -308,7 +324,7 @@ void k8s_dispatcher::handle_pod(const Json::Value& root, const msg_data& data)
 
 	if(data.m_reason == COMPONENT_ADDED)
 	{
-		Json::Value object = root["object"];
+		const Json::Value& object = root["object"];
 		if(!object.isNull())
 		{
 			if(m_state.has(m_state.get_pods(), data.m_uid))
@@ -324,7 +340,7 @@ void k8s_dispatcher::handle_pod(const Json::Value& root, const msg_data& data)
 	}
 	else if(data.m_reason == COMPONENT_MODIFIED)
 	{
-		Json::Value object = root["object"];
+		const Json::Value& object = root["object"];
 		if(!object.isNull())
 		{
 			if(!m_state.has(m_state.get_pods(), data.m_uid))
@@ -354,10 +370,14 @@ void k8s_dispatcher::handle_pod(const Json::Value& root, const msg_data& data)
 			g_logger.log(std::string("POD not found: ") + data.m_name, sinsp_logger::SEV_WARNING);
 		}
 	}
-	else // COMPONENT_ERROR
+	else if(data.m_reason == COMPONENT_ERROR)
 	{
 		log_error(root, "POD");
 	}
+	else
+	{
+		g_logger.log(std::string("Unsupported K8S POD event reason: ") + std::to_string(data.m_reason), sinsp_logger::SEV_ERROR);
+	}
 }
 
 void k8s_dispatcher::handle_rc(const Json::Value& root, const msg_data& data)
@@ -373,11 +393,12 @@ void k8s_dispatcher::handle_rc(const Json::Value& root, const msg_data& data)
 			g_logger.log(os.str(), sinsp_logger::SEV_INFO);
 		}
 		k8s_rc_t& rc = m_state.get_component<k8s_controllers, k8s_rc_t>(m_state.get_rcs(), data.m_name, data.m_uid, data.m_namespace);
-		Json::Value object = root["object"];
+		const Json::Value& object = root["object"];
 		if(!object.isNull())
 		{
 			handle_labels(rc, object["metadata"], "labels");
 			handle_selectors(rc, object["spec"], "selector");
+			rc.set_replicas(object);
 		}
 	}
 	else if(data.m_reason == COMPONENT_MODIFIED)
@@ -390,11 +411,12 @@ void k8s_dispatcher::handle_rc(const Json::Value& root, const msg_data& data)
 			return;
 		}
 		k8s_rc_t& rc = m_state.get_component<k8s_controllers, k8s_rc_t>(m_state.get_rcs(), data.m_name, data.m_uid, data.m_namespace);
-		Json::Value object = root["object"];
+		const Json::Value& object = root["object"];
 		if(!object.isNull())
 		{
 			handle_labels(rc, object["metadata"], "labels");
 			handle_selectors(rc, object["spec"], "selector");
+			rc.set_replicas(object);
 		}
 	}
 	else if(data.m_reason == COMPONENT_DELETED)
@@ -404,10 +426,14 @@ void k8s_dispatcher::handle_rc(const Json::Value& root, const msg_data& data)
 			g_logger.log(std::string("CONTROLLER not found: ") + data.m_name, sinsp_logger::SEV_ERROR);
 		}
 	}
-	else // COMPONENT_ERROR
+	else if(data.m_reason == COMPONENT_ERROR)
 	{
 		log_error(root, "REPLICATION CONTROLLER");
 	}
+	else
+	{
+		g_logger.log(std::string("Unsupported K8S REPLICATION CONTROLLER event reason: ") + std::to_string(data.m_reason), sinsp_logger::SEV_ERROR);
+	}
 }
 
 void k8s_dispatcher::handle_service(const Json::Value& root, const msg_data& data)
@@ -416,7 +442,7 @@ void k8s_dispatcher::handle_service(const Json::Value& root, const msg_data& dat
 
 	if(data.m_reason == COMPONENT_ADDED)
 	{
-		Json::Value object = root["object"];
+		const Json::Value& object = root["object"];
 		if(!object.isNull())
 		{
 			if(m_state.has(m_state.get_services(), data.m_uid))
@@ -432,7 +458,7 @@ void k8s_dispatcher::handle_service(const Json::Value& root, const msg_data& dat
 	}
 	else if(data.m_reason == COMPONENT_MODIFIED)
 	{
-		Json::Value object = root["object"];
+		const Json::Value& object = root["object"];
 		if(!object.isNull())
 		{
 			if(!m_state.has(m_state.get_services(), data.m_uid))
@@ -454,10 +480,14 @@ void k8s_dispatcher::handle_service(const Json::Value& root, const msg_data& dat
 			g_logger.log(std::string("SERVICE not found: ") + data.m_name, sinsp_logger::SEV_ERROR);
 		}
 	}
-	else // COMPONENT_ERROR
+	else if(data.m_reason == COMPONENT_ERROR)
 	{
 		log_error(root, "SERVICE");
 	}
+	else
+	{
+		g_logger.log(std::string("Unsupported K8S SERVICE event reason: ") + std::to_string(data.m_reason), sinsp_logger::SEV_ERROR);
+	}
 }
 
 void k8s_dispatcher::extract_data(const std::string& json, bool enqueue)
@@ -518,7 +548,7 @@ void k8s_dispatcher::extract_data(const std::string& json, bool enqueue)
 	}
 	else
 	{
-		g_logger.log("Bad JSON message received.", sinsp_logger::SEV_ERROR);
+		g_logger.log("Bad JSON message received :[" + json + ']', sinsp_logger::SEV_ERROR);
 	}
 }
 
