@@ -40,6 +40,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include "filter.h"
 #include "filterchecks.h"
 #include "protodecoder.h"
+#include "tracers.h"
 #ifdef HAS_ANALYZER
 #include "analyzer_int.h"
 #include "analyzer_thread.h"
@@ -51,22 +52,36 @@ bool should_drop(sinsp_evt *evt);
 extern sinsp_protodecoder_list g_decoderlist;
 extern sinsp_evttables g_infotables;
 
+#if 0
 sinsp_parser::sinsp_parser(sinsp *inspector) :
 	m_inspector(inspector),
 	m_tmp_evt(m_inspector),
 	m_fd_listener(NULL)
 {
-	m_k8s_metaevents_state.m_piscapevt = (scap_evt*)new char[SP_EVT_BUF_SIZE];
-	m_k8s_metaevents_state.m_piscapevt->type = PPME_K8S_E;
+	m_fake_userevt = (scap_evt*)m_fake_userevt_storage;
+	m_inspector->m_partial_tracers_pool = new simple_lifo_queue<sinsp_partial_tracer>(128);
 
-	m_k8s_metaevents_state.m_metaevt.m_inspector = m_inspector;
-	m_k8s_metaevents_state.m_metaevt.m_info = &(g_infotables.m_event_info[PPME_SYSDIGEVENT_X]);
-	m_k8s_metaevents_state.m_metaevt.m_pevt = NULL;
-	m_k8s_metaevents_state.m_metaevt.m_cpuid = 0;
-	m_k8s_metaevents_state.m_metaevt.m_evtnum = 0;
-	m_k8s_metaevents_state.m_metaevt.m_pevt = m_k8s_metaevents_state.m_piscapevt;
-	m_k8s_metaevents_state.m_metaevt.m_fdinfo = NULL;
+	sinsp_tracerparser p(inspector);
+	p.test();
 }
+#else
+sinsp_parser::sinsp_parser(sinsp *inspector) :
+	m_inspector(inspector),
+	m_tmp_evt(m_inspector),
+	m_fd_listener(NULL)
+{
+	m_fake_userevt = (scap_evt*)m_fake_userevt_storage;
+
+	//
+	// Note: allocated here instead of in the sinsp constructor because sinsp_partial_tracer
+	//       is not defined in sinsp.cpp
+	//
+	m_inspector->m_partial_tracers_pool = new simple_lifo_queue<sinsp_partial_tracer>(128);
+
+	init_metaevt(m_k8s_metaevents_state, PPME_K8S_E, SP_EVT_BUF_SIZE);
+	init_metaevt(m_mesos_metaevents_state, PPME_MESOS_E, SP_EVT_BUF_SIZE);
+}
+#endif
 
 sinsp_parser::~sinsp_parser()
 {
@@ -75,8 +90,40 @@ sinsp_parser::~sinsp_parser()
 		delete m_protodecoders[j];
 	}
 
+	while(!m_tmp_events_buffer.empty())
+	{
+		auto ptr = m_tmp_events_buffer.top();
+		free(ptr);
+		m_tmp_events_buffer.pop();
+	}
 	m_protodecoders.clear();
-	delete[] m_k8s_metaevents_state.m_piscapevt;
+
+	free(m_k8s_metaevents_state.m_piscapevt);
+	free(m_mesos_metaevents_state.m_piscapevt);
+
+	if(m_inspector->m_partial_tracers_pool != NULL)
+	{
+		delete m_inspector->m_partial_tracers_pool;
+	}
+}
+
+void sinsp_parser::init_scapevt(metaevents_state& evt_state, uint16_t evt_type, uint16_t buf_size)
+{
+	evt_state.m_piscapevt = (scap_evt*) realloc(evt_state.m_piscapevt, buf_size);
+	evt_state.m_scap_buf_size = buf_size;
+	evt_state.m_piscapevt->type = evt_type;
+	evt_state.m_metaevt.m_pevt = evt_state.m_piscapevt;
+}
+
+void sinsp_parser::init_metaevt(metaevents_state& evt_state, uint16_t evt_type, uint16_t buf_size)
+{
+	evt_state.m_piscapevt = 0;
+	init_scapevt(evt_state, evt_type, buf_size);
+	evt_state.m_metaevt.m_inspector = m_inspector;
+	evt_state.m_metaevt.m_info = &(g_infotables.m_event_info[PPME_SYSDIGEVENT_X]);
+	evt_state.m_metaevt.m_cpuid = 0;
+	evt_state.m_metaevt.m_evtnum = 0;
+	evt_state.m_metaevt.m_fdinfo = NULL;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -121,7 +168,37 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 
 	if(m_inspector->m_filter)
 	{
-		ppm_event_flags eflags = evt->get_flags();
+		ppm_event_flags eflags = evt->get_info_flags();
+
+		if(etype == PPME_SYSCALL_WRITE_X)
+		{
+			//
+			// Check if this is a user event
+			//
+			sinsp_fdinfo_t* fdinfo = evt->m_fdinfo;
+
+			if(fdinfo == NULL)
+			{
+				fdinfo = evt->m_tinfo->get_fd(evt->m_tinfo->m_lastevent_fd);
+				evt->m_fdinfo = fdinfo;
+			}
+
+			if(fdinfo && fdinfo->m_flags & sinsp_fdinfo_t::FLAGS_IS_TRACER_FD)
+			{
+				eflags = (ppm_event_flags)(((uint64_t)eflags) | EF_MODIFIES_STATE);
+			}
+			else
+			{
+				if(!m_inspector->m_islive)
+				{
+					if((evt->get_dump_flags() & SCAP_DF_TRACER) != 0)
+					{
+						evt->m_fdinfo = NULL;
+						eflags = (ppm_event_flags)(((uint64_t)eflags) | EF_MODIFIES_STATE);
+					}
+				}
+			}
+		}
 
 		if(eflags & EF_MODIFIES_STATE)
 		{
@@ -135,7 +212,7 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 				{
 					if(!(eflags & EF_SKIPPARSERESET || etype == PPME_SCHEDSWITCH_6_E))
 					{
-						evt->m_tinfo->m_lastevent_type = PPM_SC_MAX;
+						evt->m_tinfo->m_lastevent_type = PPM_EVENT_MAX;
 					}
 				}
 
@@ -173,6 +250,20 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 	case PPME_SYSCALL_SETGID_E:
 		store_event(evt);
 		break;
+	case PPME_SYSCALL_WRITE_E:
+		if(!m_inspector->m_dumper)
+		{
+			evt->m_fdinfo = evt->m_tinfo->get_fd(evt->m_tinfo->m_lastevent_fd);
+			if(evt->m_fdinfo)
+			{
+				if(evt->m_fdinfo->m_flags & sinsp_fdinfo_t::FLAGS_IS_TRACER_FD)
+				{
+					evt->m_filtered_out = true;
+					return;
+				}
+			}
+		}
+		break;
 	case PPME_SYSCALL_READ_X:
 	case PPME_SYSCALL_WRITE_X:
 	case PPME_SOCKET_RECV_X:
@@ -316,7 +407,10 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 		parse_setgid_exit(evt);
 		break;
 	case PPME_CONTAINER_E:
-		parse_container_evt(evt);
+		parse_container_evt(evt); // deprecated, only here for backwards compatibility
+		break;
+	case PPME_CONTAINER_JSON_E:
+		parse_container_json_evt(evt);
 		break;
 	case PPME_CPU_HOTPLUG_E:
 		parse_cpu_hotplug_enter(evt);
@@ -327,6 +421,12 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 			parse_k8s_evt(evt);
 		}
 		break;
+	case PPME_MESOS_E:
+		if(!m_inspector->is_live())
+		{
+			parse_mesos_evt(evt);
+		}
+		break;
 	case PPME_SYSCALL_CHROOT_X:
 		parse_chroot_exit(evt);
 		break;
@@ -353,7 +453,7 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 	}
 #endif
 	//
-	// Offline captures can prodice events with the SCAP_DF_STATE_ONLY. They are
+	// Offline captures can produce events with the SCAP_DF_STATE_ONLY. They are
 	// supposed to go through the engine, but they must be filtered out before 
 	// reaching the user.
 	//
@@ -392,7 +492,7 @@ bool sinsp_parser::reset(sinsp_evt *evt)
 	//
 	evt->init();
 
-	ppm_event_flags eflags = evt->get_flags();
+	ppm_event_flags eflags = evt->get_info_flags();
 	uint16_t etype = evt->get_type();
 
 	evt->m_fdinfo = NULL;
@@ -524,7 +624,11 @@ bool sinsp_parser::reset(sinsp_evt *evt)
 		else
 		{
 			tinfo->set_lastevent_data_validity(false);
-			return false;
+
+			if(tinfo->m_lastevent_type != PPME_TRACER_E)
+			{
+				return false;
+			}
 		}
 
 		//
@@ -624,7 +728,10 @@ void sinsp_parser::store_event(sinsp_evt *evt)
 	// Copy the data
 	//
 	auto tinfo = evt->m_tinfo;
-	tinfo->m_lastevent_data = reserve_event_buffer();
+	if(tinfo->m_lastevent_data == NULL)
+	{
+		tinfo->m_lastevent_data = reserve_event_buffer();
+	}
 	memcpy(tinfo->m_lastevent_data, evt->m_pevt, elen);
 	tinfo->m_lastevent_cpuid = evt->get_cpuid();
 
@@ -1494,12 +1601,13 @@ void sinsp_parser::parse_openat_dir(sinsp_evt *evt, char* name, int64_t dirfd, O
 	}
 }
 
-void schedule_more_k8s_evts(sinsp* inspector, void* data)
+template <typename T>
+void schedule_more_evts(sinsp* inspector, void* data, T* client, ppm_event_type evt_type)
 {
 #ifdef HAS_CAPTURE
 	ASSERT(data);
 	bool good_event = false;
-	k8s_metaevents_state* state = (k8s_metaevents_state*)data;
+	metaevents_state* state = (metaevents_state*)data;
 
 	if(state->m_new_group == true)
 	{
@@ -1508,37 +1616,34 @@ void schedule_more_k8s_evts(sinsp* inspector, void* data)
 		return;
 	}
 
-	k8s* k8s_client = inspector->get_k8s_client();
-	ASSERT(k8s_client);
-	if(!k8s_client->get_capture_events().size())
+	ASSERT(client);
+	if(!client->get_capture_events().size())
 	{
-		g_logger.log("K8S event scheduled but no events available."
-					"All pending K8S event request are cancelled.", sinsp_logger::SEV_ERROR);
+		g_logger.log(std::string("An event scheduled but no events available."
+					"All pending event requests for "
+					"[") + typeid(T).name() + "] are cancelled.", sinsp_logger::SEV_ERROR);
 		state->m_new_group = false;
-		state->m_n_additional_k8s_events_to_add = 0;
+		state->m_n_additional_events_to_add = 0;
 		inspector->remove_meta_event_callback();
 		return;
 	}
-	string payload = k8s_client->dequeue_capture_event();
+	string payload = client->dequeue_capture_event();
 	std::size_t tot_len = sizeof(scap_evt) + sizeof(uint16_t) + payload.size() + 1;
 
-	if(tot_len <= SP_EVT_BUF_SIZE)
+	if(tot_len > state->m_scap_buf_size)
 	{
-		state->m_piscapevt->len = tot_len;
-		uint16_t* plen = (uint16_t*)((char *)state->m_piscapevt + sizeof(struct ppm_evt_hdr));
-		plen[0] = (uint16_t)payload.size() + 1;
-		uint8_t* edata = (uint8_t*)plen + sizeof(uint16_t);
-		memcpy(edata, payload.c_str(), plen[0]);
-		good_event = true;
-	}
-	else
-	{
-		g_logger.log("K8S event larger than available buffer, will not be recorded. "
-					"This may result in an inaccurate K8S event log.", sinsp_logger::SEV_ERROR);
+		sinsp_parser::init_scapevt(*state, evt_type, tot_len);
 	}
 
-	state->m_n_additional_k8s_events_to_add--;
-	if(state->m_n_additional_k8s_events_to_add == 0)
+	state->m_piscapevt->len = tot_len;
+	uint16_t* plen = (uint16_t*)((char *)state->m_piscapevt + sizeof(struct ppm_evt_hdr));
+	plen[0] = (uint16_t)payload.size() + 1;
+	uint8_t* edata = (uint8_t*)plen + sizeof(uint16_t);
+	memcpy(edata, payload.c_str(), plen[0]);
+	good_event = true;
+
+	state->m_n_additional_events_to_add--;
+	if(state->m_n_additional_events_to_add == 0)
 	{
 		inspector->remove_meta_event_callback();
 	}
@@ -1549,6 +1654,11 @@ void schedule_more_k8s_evts(sinsp* inspector, void* data)
 #endif // HAS_CAPTURE
 }
 
+void schedule_more_k8s_evts(sinsp* inspector, void* data)
+{
+	schedule_more_evts(inspector, data, inspector->get_k8s_client(), PPME_K8S_E);
+}
+
 void sinsp_parser::schedule_k8s_events(sinsp_evt *evt)
 {
 #ifdef HAS_CAPTURE
@@ -1564,7 +1674,7 @@ void sinsp_parser::schedule_k8s_events(sinsp_evt *evt)
 			m_k8s_metaevents_state.m_piscapevt->tid = evt->get_tid();
 			m_k8s_metaevents_state.m_piscapevt->ts = m_inspector->m_lastevent_ts;
 			m_k8s_metaevents_state.m_new_group = true;
-			m_k8s_metaevents_state.m_n_additional_k8s_events_to_add = event_count;
+			m_k8s_metaevents_state.m_n_additional_events_to_add = event_count;
 			m_inspector->add_meta_event_callback(&schedule_more_k8s_evts, &m_k8s_metaevents_state);
 
 			schedule_more_k8s_evts(m_inspector, &m_k8s_metaevents_state);
@@ -1573,6 +1683,35 @@ void sinsp_parser::schedule_k8s_events(sinsp_evt *evt)
 #endif // HAS_CAPTURE
 }
 
+void schedule_more_mesos_evts(sinsp* inspector, void* data)
+{
+	schedule_more_evts(inspector, data, inspector->get_mesos_client(), PPME_MESOS_E);
+}
+
+void sinsp_parser::schedule_mesos_events(sinsp_evt *evt)
+{
+#ifdef HAS_CAPTURE
+	//
+	// schedule mesos events, if any available
+	//
+	mesos* mesos_client = 0;
+	if(m_inspector && (mesos_client = m_inspector->m_mesos_client))
+	{
+		int event_count = mesos_client->get_capture_events().size();
+		if(event_count)
+		{
+			m_mesos_metaevents_state.m_piscapevt->tid = evt->get_tid();
+			m_mesos_metaevents_state.m_piscapevt->ts = m_inspector->m_lastevent_ts;
+			m_mesos_metaevents_state.m_new_group = true;
+			m_mesos_metaevents_state.m_n_additional_events_to_add = event_count;
+			m_inspector->add_meta_event_callback(&schedule_more_mesos_evts, &m_mesos_metaevents_state);
+
+			schedule_more_mesos_evts(m_inspector, &m_mesos_metaevents_state);
+		}
+	}
+#endif // HAS_CAPTURE
+}
+
 void sinsp_parser::parse_open_openat_creat_exit(sinsp_evt *evt)
 {
 	sinsp_evt_param *parinfo;
@@ -1673,6 +1812,14 @@ void sinsp_parser::parse_open_openat_creat_exit(sinsp_evt *evt)
 		fdi.m_openflags = flags;
 		fdi.add_filename(fullpath);
 
+		//
+		// If this is a user event fd, mark it with the proper flag
+		//
+		if(fdi.m_name == USER_EVT_DEVICE_NAME)
+		{
+			fdi.m_flags |= sinsp_fdinfo_t::FLAGS_IS_TRACER_FD;
+		}
+
 		//
 		// Add the fd to the table.
 		//
@@ -2590,18 +2737,115 @@ void sinsp_parser::swap_ipv4_addresses(sinsp_fdinfo_t* fdinfo)
 	fdinfo->m_sockinfo.m_ipv4info.m_fields.m_dport = tport;
 }
 
+void sinsp_parser::parse_tracer(sinsp_evt *evt, int64_t retval)
+{
+	sinsp_threadinfo* tinfo = evt->m_tinfo;
+	ASSERT(tinfo);
+
+	//
+	// Extract the data buffer
+	//
+	sinsp_evt_param *parinfo = evt->get_param(1);
+	char* data = parinfo->m_val;
+	uint32_t datalen = parinfo->m_len;
+	sinsp_tracerparser* p = tinfo->m_tracer_parser;
+
+	if(p == NULL)
+	{
+		p = tinfo->m_tracer_parser = new sinsp_tracerparser(m_inspector);
+	}
+
+	p->m_tinfo = tinfo;
+
+	p->process_event_data(data, datalen, evt->get_ts());
+
+	if(p->m_res == sinsp_tracerparser::RES_TRUNCATED)
+	{
+		if(!m_inspector->m_dumper)
+		{
+			evt->m_filtered_out = true;
+		}
+		return;
+	}
+
+	p->m_args.first = &p->m_argnames;
+	p->m_args.second = &p->m_argvals;
+
+	//
+	// Populate the user event that we will send up the stack instead of the write
+	//
+	uint8_t* fakeevt_storage = (uint8_t*)m_fake_userevt;
+	m_fake_userevt->ts = evt->m_pevt->ts;
+	m_fake_userevt->tid = evt->m_pevt->tid;
+
+	if(p->m_res == sinsp_tracerparser::RES_OK)
+	{
+		if(p->m_type_str[0] == '>')
+		{
+			m_fake_userevt->type = PPME_TRACER_E;
+		}
+		else
+		{
+			m_fake_userevt->type = PPME_TRACER_X;
+		}
+
+		uint16_t *lens = (uint16_t *)(fakeevt_storage + sizeof(struct ppm_evt_hdr));
+		lens[0] = 8;
+		lens[1] = 8;
+		lens[2] = 8;
+
+		*(uint64_t *)(fakeevt_storage + sizeof(struct ppm_evt_hdr) + 6) = p->m_id;
+		*(uint64_t *)(fakeevt_storage + sizeof(struct ppm_evt_hdr) + 14) = (uint64_t)&p->m_tags;
+		*(uint64_t *)(fakeevt_storage + sizeof(struct ppm_evt_hdr) + 22) = (uint64_t)&p->m_args;
+	}
+	else
+	{
+		//
+		// Parsing error.
+		// We don't know the direction, so we use enter.
+		//
+		p->m_argnames.clear();
+		p->m_argvals.clear();
+
+		m_fake_userevt->type = PPME_TRACER_E;
+
+		uint16_t *lens = (uint16_t *)(fakeevt_storage + sizeof(struct ppm_evt_hdr));
+		lens[0] = 8;
+		lens[1] = 8;
+		lens[2] = 8;
+
+		p->m_tags.clear();
+		m_tracer_error_string = "invalid tracer " + string(data, datalen) + ", len" + to_string(datalen);
+		p->m_tags.push_back((char*)m_tracer_error_string.c_str());
+		*(uint64_t *)(fakeevt_storage + sizeof(struct ppm_evt_hdr) + 6) = 0;
+		*(uint64_t *)(fakeevt_storage + sizeof(struct ppm_evt_hdr) + 14) = (uint64_t)&p->m_tags;
+		*(uint64_t *)(fakeevt_storage + sizeof(struct ppm_evt_hdr) + 22) = (uint64_t)&p->m_args;
+	}
+
+	scap_evt* tevt = evt->m_pevt;
+	evt->m_pevt = m_fake_userevt;
+	evt->init();
+	evt->m_poriginal_evt = tevt;
+	evt->m_flags |= (uint32_t)sinsp_evt::SINSP_EF_IS_TRACER;
+
+	//
+	// Update some thread information
+	//
+	tinfo->m_lastevent_fd = -1;
+	tinfo->m_lastevent_type = PPME_TRACER_E;
+	tinfo->m_latency = 0;
+	tinfo->m_last_latency_entertime = 0;
+
+	return;
+}
+
 void sinsp_parser::parse_rw_exit(sinsp_evt *evt)
 {
 	sinsp_evt_param *parinfo;
 	int64_t retval;
 	int64_t tid = evt->get_tid();
 	sinsp_evt *enter_evt = &m_tmp_evt;
-	ppm_event_flags eflags = evt->get_flags();
-
-	if(!evt->m_fdinfo)
-	{
-		return;
-	}
+	ppm_event_flags eflags = evt->get_info_flags();
 
 	//
 	// Extract the return value
@@ -2610,6 +2854,30 @@ void sinsp_parser::parse_rw_exit(sinsp_evt *evt)
 	ASSERT(parinfo->m_len == sizeof(int64_t));
 	retval = *(int64_t *)parinfo->m_val;
 
+	if(evt->m_fdinfo == NULL)
+	{
+		if(!m_inspector->m_islive)
+		{
+			if((evt->get_dump_flags() & SCAP_DF_TRACER) != 0)
+			{
+				parse_tracer(evt, retval);
+				return;
+			}
+		}
+
+		return;
+	}
+
+	//
+	// User events get into the engine as normal writes, but the FD has a flag to
+	// quickly recognize them.
+	//
+	if(evt->m_fdinfo->m_flags & sinsp_fdinfo_t::FLAGS_IS_TRACER_FD)
+	{
+		parse_tracer(evt, retval);
+		return;
+	}
+
 	//
 	// If the operation was successful, validate that the fd exists
 	//
@@ -3240,7 +3508,7 @@ void sinsp_parser::parse_getrlimit_setrlimit_exit(sinsp_evt *evt)
 			{
 				if(evt->m_tinfo->get_main_thread()->m_fdlimit != -1)
 				{
-					ASSERT(curval == evt->m_tinfo->get_main_thread()->m_fdlimit);
+//					ASSERT(curval == evt->m_tinfo->get_main_thread()->m_fdlimit);
 				}
 			}
 #endif
@@ -3335,7 +3603,10 @@ void sinsp_parser::parse_select_poll_epollwait_enter(sinsp_evt *evt)
 		return;
 	}
 
-	evt->m_tinfo->m_lastevent_data = reserve_event_buffer();
+	if(evt->m_tinfo->m_lastevent_data == NULL)
+	{
+		evt->m_tinfo->m_lastevent_data = reserve_event_buffer();
+	}
 	*(uint64_t*)evt->m_tinfo->m_lastevent_data = evt->get_ts();
 }
 void sinsp_parser::parse_fcntl_enter(sinsp_evt *evt)
@@ -3543,6 +3814,59 @@ void sinsp_parser::parse_setgid_exit(sinsp_evt *evt)
 	}
 }
 
+void sinsp_parser::parse_container_json_evt(sinsp_evt *evt)
+{
+	sinsp_evt_param *parinfo = evt->get_param(0);
+	ASSERT(parinfo);
+	ASSERT(parinfo->m_len > 0);
+	std::string json(parinfo->m_val, parinfo->m_len);
+	g_logger.log(json, sinsp_logger::SEV_DEBUG);
+	ASSERT(m_inspector);
+	Json::Value root;
+	if(Json::Reader().parse(json, root))
+	{
+		sinsp_container_info container_info;
+		const Json::Value& container = root["container"];
+		const Json::Value& id = container["id"];
+		if(!id.isNull() && id.isConvertibleTo(Json::stringValue))
+		{
+			container_info.m_id = id.asString();
+		}
+		const Json::Value& type = container["type"];
+		if(!type.isNull() && type.isConvertibleTo(Json::uintValue))
+		{
+			container_info.m_type = static_cast<sinsp_container_type>(type.asUInt());
+		}
+		const Json::Value& name = container["name"];
+		if(!name.isNull() && name.isConvertibleTo(Json::stringValue))
+		{
+			container_info.m_name = name.asString();
+		}
+		const Json::Value& image = container["image"];
+		if(!image.isNull() && image.isConvertibleTo(Json::stringValue))
+		{
+			container_info.m_image = image.asString();
+		}
+		const Json::Value& mesos_task_id = container["mesos_task_id"];
+		if(!mesos_task_id.isNull() && mesos_task_id.isConvertibleTo(Json::stringValue))
+		{
+			container_info.m_mesos_task_id = mesos_task_id.asString();
+		}
+		m_inspector->m_container_manager.add_container(container_info);
+		/*
+		g_logger.log("Container\n-------\nID:" + container_info.m_id +
+					 "\nType: " + std::to_string(container_info.m_type) +
+					 "\nName: " + container_info.m_name +
+					 "\nImage: " + container_info.m_image +
+					 "\nMesos Task ID: " + container_info.m_mesos_task_id, sinsp_logger::SEV_DEBUG);
+		*/
+	}
+	else
+	{
+		g_logger.log("Invalid JSON encountered while parsing container info: " + json, sinsp_logger::SEV_ERROR);
+	}
+}
+
 void sinsp_parser::parse_container_evt(sinsp_evt *evt)
 {
 	sinsp_evt_param *parinfo;
@@ -3600,6 +3924,18 @@ void sinsp_parser::parse_k8s_evt(sinsp_evt *evt)
 	m_inspector->m_k8s_client->simulate_watch_event(json);
 }
 
+void sinsp_parser::parse_mesos_evt(sinsp_evt *evt)
+{
+	sinsp_evt_param *parinfo = evt->get_param(0);
+	ASSERT(parinfo);
+	ASSERT(parinfo->m_len > 0);
+	std::string json(parinfo->m_val, parinfo->m_len);
+	//g_logger.log(json, sinsp_logger::SEV_DEBUG);
+	ASSERT(m_inspector);
+	ASSERT(m_inspector->m_mesos_client);
+	m_inspector->m_mesos_client->simulate_event(json);
+}
+
 void sinsp_parser::parse_chroot_exit(sinsp_evt *evt)
 {
 	auto parinfo = evt->get_param(0);
@@ -3632,4 +3968,4 @@ void sinsp_parser::free_event_buffer(uint8_t *ptr)
 	{
 		free(ptr);
 	}
-}
\ No newline at end of file
+}
