@@ -96,6 +96,7 @@ sinsp::sinsp() :
 	m_output_time_flag = 'h';
 	m_max_evt_output_len = 0;
 	m_filesize = -1;
+	m_track_tracers_state = false;
 	m_import_users = true;
 	m_meta_evt_buf = new char[SP_EVT_BUF_SIZE];
 	m_meta_evt.m_pevt = (scap_evt*) m_meta_evt_buf;
@@ -137,6 +138,9 @@ sinsp::sinsp() :
 	m_k8s_api_server = NULL;
 	m_k8s_api_cert = NULL;
 
+	m_mesos_client = NULL;
+	m_mesos_last_watch_time_ns = 0;
+
 	m_filter_proc_table_when_saving = false;
 }
 
@@ -181,6 +185,8 @@ sinsp::~sinsp()
 	delete m_k8s_client;
 	delete m_k8s_api_server;
 	delete m_k8s_api_cert;
+
+	delete m_mesos_client;
 }
 
 void sinsp::add_protodecoders()
@@ -248,6 +254,15 @@ void sinsp::init()
 	m_n_proc_lookups = 0;
 	m_n_proc_lookups_duration_ns = 0;
 
+	//
+	// Return the tracers to the pool and clear the tracers list
+	//
+	for(auto it = m_partial_tracers_list.begin(); it != m_partial_tracers_list.end(); ++it)
+	{
+		m_partial_tracers_pool->push(*it);
+	}
+	m_partial_tracers_list.clear();
+
 	//
 	// If we're reading from file, we try to pre-parse the container events before
 	// importing the thread table, so that thread table filtering will work with
@@ -269,7 +284,7 @@ void sinsp::init()
 
 			if(res == SCAP_SUCCESS)
 			{
-				if(pevent->type != PPME_CONTAINER_E)
+				if((pevent->type != PPME_CONTAINER_E) && (pevent->type != PPME_CONTAINER_JSON_E))
 				{
 					break;
 				}
@@ -289,7 +304,6 @@ void sinsp::init()
 		// Rewind and consume the exact number of events
 		//
 		scap_fseek(m_h, off);
-
 		for(uint32_t j = 0; j < ncnt; j++)
 		{
 			sinsp_evt* tevt;
@@ -652,7 +666,7 @@ void sinsp::import_thread_table()
 
 void sinsp::import_ifaddr_list()
 {
-	m_network_interfaces = new sinsp_network_interfaces;
+	m_network_interfaces = new sinsp_network_interfaces(this);
 	m_network_interfaces->import_interfaces(scap_get_ifaddr_list(m_h));
 }
 
@@ -922,11 +936,16 @@ int32_t sinsp::next(OUT sinsp_evt **puevt)
 	{
 		m_thread_manager->remove_inactive_threads();
 		m_container_manager.remove_inactive_containers();
-		
+
 		if(m_k8s_client)
 		{
 			update_kubernetes_state();
 		}
+
+		if(m_mesos_client)
+		{
+			update_mesos_state();
+		}
 	}
 #endif // HAS_ANALYZER
 
@@ -1030,7 +1049,9 @@ int32_t sinsp::next(OUT sinsp_evt **puevt)
 			}
 		}
 
-		res = scap_dump(m_h, m_dumper, evt->m_pevt, evt->m_cpuid, dflags);
+		scap_evt* pdevt = (evt->m_poriginal_evt)? evt->m_poriginal_evt : evt->m_pevt;
+
+		res = scap_dump(m_h, m_dumper, pdevt, evt->m_cpuid, dflags);
 
 		if(SCAP_SUCCESS != res)
 		{
@@ -1282,6 +1303,17 @@ void sinsp::start_dropping_mode(uint32_t sampling_ratio)
 }
 
 #ifdef HAS_FILTERING
+void sinsp::set_filter(sinsp_filter* filter)
+{
+	if(m_filter != NULL)
+	{
+		ASSERT(false);
+		throw sinsp_exception("filter can only be set once");
+	}
+
+	m_filter = filter;
+}
+
 void sinsp::set_filter(const string& filter)
 {
 	if(m_filter != NULL)
@@ -1290,7 +1322,8 @@ void sinsp::set_filter(const string& filter)
 		throw sinsp_exception("filter can only be set once");
 	}
 
-	m_filter = new sinsp_filter(this, filter);
+	sinsp_filter_compiler compiler(this, filter);
+	m_filter = compiler.compile();
 	m_filterstring = filter;
 }
 
@@ -1522,24 +1555,124 @@ bool sinsp::remove_inactive_threads()
 	return m_thread_manager->remove_inactive_threads();
 }
 
-void sinsp::init_k8s_client(string* api_server, string* ssl_cert)
+void sinsp::init_mesos_client(string* api_server, bool verbose)
+{
+	m_verbose_json = verbose;
+	if(m_mesos_client == NULL)
+	{
+		if(api_server)
+		{
+			// -m <url[,marathon_url]>
+			std::string::size_type pos = api_server->find(',');
+			if(pos != std::string::npos)
+			{
+				m_marathon_api_server.clear();
+				m_marathon_api_server.push_back(api_server->substr(pos + 1));
+			}
+			m_mesos_api_server = api_server->substr(0, pos);
+		}
+
+		bool is_live = !m_mesos_api_server.empty();
+		m_mesos_client = new mesos(m_mesos_api_server, mesos::default_state_api,
+									m_marathon_api_server,
+									mesos::default_groups_api,
+									mesos::default_apps_api,
+									false, // no leader follow
+									mesos::default_timeout_ms,
+									is_live,
+									m_verbose_json);
+	}
+}
+
+void sinsp::init_k8s_client(string* api_server, string* ssl_cert, bool verbose)
 {
 	ASSERT(api_server);
+	m_verbose_json = verbose;
 	m_k8s_api_server = api_server;
 	m_k8s_api_cert = ssl_cert;
 
 	if(m_k8s_client == NULL)
 	{
-		g_logger.log("Fetching initial k8s state", sinsp_logger::SEV_INFO);
+#ifdef HAS_CAPTURE
+		std::shared_ptr<sinsp_curl::ssl> k8s_ssl;
+		std::shared_ptr<sinsp_curl::bearer_token> k8s_bt;
+
+		if(ssl_cert)
+		{
+			std::string cert;
+			std::string key;
+			std::string key_pwd;
+			std::string ca_cert;
+
+			// -K <bt_file> | <cert_file>:<key_file[#password]>[:<ca_cert_file>]
+			std::string::size_type pos = ssl_cert->find(':');
+			if(pos == std::string::npos) // ca_cert-only is obsoleted, single entry is now bearer token
+			{
+				k8s_bt = std::make_shared<sinsp_curl::bearer_token>(*ssl_cert);
+				ssl_cert->clear();
+			}
+			else
+			{
+				while(ssl_cert->length())
+				{
+					if(cert.empty() && pos != std::string::npos)
+					{
+						cert = ssl_cert->substr(0, pos);
+						if(ssl_cert->length() > (pos + 1))
+						{
+							*ssl_cert = ssl_cert->substr(pos + 1);
+						}
+						else { break; }
+					}
+					else if(key.empty())
+					{
+						key = ssl_cert->substr(0, pos);
+						if(ssl_cert->length() > (pos + 1))
+						{
+							*ssl_cert = ssl_cert->substr(pos + 1);
+							std::string::size_type s_pos = key.find('#');
+							if(s_pos != std::string::npos && key.length() > (s_pos + 1))
+							{
+								key_pwd = key.substr(s_pos + 1);
+								key = key.substr(0, s_pos);
+							}
+							if(pos == std::string::npos) { break; }
+						}
+						else { break; }
+					}
+					else if(ca_cert.empty())
+					{
+						ca_cert = *ssl_cert;
+						ssl_cert->clear();
+					}
+					else { goto ssl_err; }
+					pos = ssl_cert->find(':', pos);
+				}
+				if(cert.empty() || key.empty()) { goto ssl_err; }
+			}
+			k8s_ssl = std::make_shared<sinsp_curl::ssl>(cert, key, key_pwd,
+						ca_cert, ca_cert.empty() ? false : true, "PEM");
+		}
+#endif // HAS_CAPTURE
 		bool is_live = !m_k8s_api_server->empty();
 		m_k8s_client = new k8s(*m_k8s_api_server,
 			is_live ? true : false, // watch
 			false, // don't run watch in thread
 			is_live ? true : false, // capture
-			"/api/v1/",
-			m_k8s_api_cert ? *m_k8s_api_cert : string("")
+			"/api/v1"
+#ifdef HAS_CAPTURE
+			,k8s_ssl
+			,k8s_bt
+#endif // HAS_CAPTURE
 		);
 	}
+
+	return;
+
+#ifdef HAS_CAPTURE
+ssl_err:
+	throw sinsp_exception(string("Invalid K8S SSL entry: ") + (ssl_cert ? *ssl_cert : string("NULL")));
+#endif // HAS_CAPTURE
 }
 
 void sinsp::update_kubernetes_state()
@@ -1549,23 +1682,83 @@ void sinsp::update_kubernetes_state()
 	{
 		m_k8s_last_watch_time_ns = m_lastevent_ts;
 
-		if(m_k8s_client->is_alive())
+		if(m_parser && m_k8s_client->is_alive())
 		{
 			uint64_t delta = sinsp_utils::get_current_time_ns();
 
 			m_k8s_client->watch();
-			this->m_parser->schedule_k8s_events(&m_meta_evt);
+			m_parser->schedule_k8s_events(&m_meta_evt);
 
 			delta = sinsp_utils::get_current_time_ns() - delta;
 
-			g_logger.format(sinsp_logger::SEV_INFO, "Updating Kubernetes state took %" PRIu64 " ms", delta / 1000000LL);
+			g_logger.format(sinsp_logger::SEV_DEBUG, "Updating Kubernetes state took %" PRIu64 " ms", delta / 1000000LL);
 		}
 		else
 		{
 			g_logger.format(sinsp_logger::SEV_WARNING, "Kubernetes connection not active anymore, retrying");
 			delete m_k8s_client;
 			m_k8s_client = NULL;
-			init_k8s_client(m_k8s_api_server, m_k8s_api_cert);
+			init_k8s_client(m_k8s_api_server, m_k8s_api_cert, m_verbose_json);
+		}
+	}
+}
+
+bool sinsp::get_mesos_data()
+{
+	bool ret = false;
+#ifdef HAS_CAPTURE
+	try
+	{
+		static time_t last_mesos_refresh = 0;
+		ASSERT(m_mesos_client);
+		ASSERT(m_mesos_client->is_alive());
+
+		time_t now; time(&now);
+		if(last_mesos_refresh)
+		{
+			g_logger.log("Collecting Mesos data ...", sinsp_logger::SEV_DEBUG);
+			ret = m_mesos_client->collect_data();
+		}
+		if(difftime(now, last_mesos_refresh) > 10)
+		{
+			g_logger.log("Requesting Mesos data ...", sinsp_logger::SEV_DEBUG);
+			m_mesos_client->send_data_request(false);
+			last_mesos_refresh = now;
+		}
+	}
+	catch(std::exception& ex)
+	{
+		g_logger.log(std::string("Mesos exception: ") + ex.what(), sinsp_logger::SEV_ERROR);
+		delete m_mesos_client;
+		m_mesos_client = NULL;
+		init_mesos_client(0, m_verbose_json);
+	}
+#endif // HAS_CAPTURE
+	return ret;
+}
+
+void sinsp::update_mesos_state()
+{
+	ASSERT(m_mesos_client);
+	if(m_lastevent_ts > m_mesos_last_watch_time_ns + ONE_SECOND_IN_NS)
+	{
+		m_mesos_last_watch_time_ns = m_lastevent_ts;
+		if(m_mesos_client->is_alive())
+		{
+			uint64_t delta = sinsp_utils::get_current_time_ns();
+			if(m_parser && get_mesos_data())
+			{
+				m_parser->schedule_mesos_events(&m_meta_evt);
+				delta = sinsp_utils::get_current_time_ns() - delta;
+				g_logger.format(sinsp_logger::SEV_DEBUG, "Updating Mesos state took %" PRIu64 " ms", delta / 1000000LL);
+			}
+		}
+		else
+		{
+			g_logger.format(sinsp_logger::SEV_ERROR, "Mesos connection not active anymore, retrying ...");
+			delete m_mesos_client;
+			m_mesos_client = NULL;
+			init_mesos_client(0, m_verbose_json);
 		}
 	}
 }
