@@ -25,6 +25,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 class sinsp_delays_info;
 class sinsp_threadtable_listener;
 class thread_analyzer_info;
+class sinsp_tracerparser;
 
 typedef struct erase_fd_params
 {
@@ -73,6 +74,15 @@ public:
 	*/
 	string get_cwd();
 
+	/*!
+	  \brief Return the values of all environment variables for the process
+	  containing this thread.
+	*/
+	const vector<string>& get_env() const
+	{
+		return m_env;
+	}
+
 	/*!
 	  \brief Return the value of the specified environment variable for the process
 	  containing this thread. Returns empty string if variable is not found.
@@ -223,6 +233,11 @@ public:
 	uint64_t m_lastaccess_ts; ///< The last time this thread was looked up. Used when cleaning up the table. 
 	uint64_t m_clone_ts; ///< When the clone that started this process happened.
 
+	//
+	// Parser for the user events. Public so that filter fields can access it
+	//
+	sinsp_tracerparser* m_tracer_parser;
+
 	thread_analyzer_info* m_ainfo;
 
 #ifdef HAS_FILTERING
@@ -313,6 +328,7 @@ VISIBILITY_PRIVATE
 	friend class sinsp_thread_manager;
 	friend class sinsp_transaction_table;
 	friend class thread_analyzer_info;
+	friend class sinsp_tracerparser;
 	friend class lua_cbacks;
 };
 
