@@ -46,7 +46,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include "cursestable.h"
 #include "cursesui.h"
 
-#define MOUSE_CAPABLE_TERM "xterm-1002"
+#define MOUSE_CAPABLE_TERM "xterm-1003"
 
 static bool g_terminate = false;
 static void usage();
@@ -80,16 +80,23 @@ static void usage()
 "                    increase sysdig's startup time. Moreover, they contain\n"
 "                    information that could be privacy sensitive.\n"
 " -h, --help         Print this page\n"
-" -k, --k8s-api      Enable Kubernetes support by connecting to the API server\n"
+" -k <url>, --k8s-api=<url>\n"
+"                    Enable Kubernetes support by connecting to the API server\n"
 "                    specified as argument. E.g. \"http://admin:password@127.0.0.1:8080\".\n"
 "                    The API server can also be specified via the environment variable\n"
 "                    SYSDIG_K8S_API.\n"
-" -K, --k8s-api-cert=<file_name>\n"
-"                    Use the provided certificate file name to authenticate with the K8S API server.\n"
-"                    Filename must be a full absolute or relative (to the current directory) path\n"
-"                    to the certificate file.\n"
-"                    The certificate can also be specified via the environment variable.\n"
-"                    SYSDIG_K8S_API_CERT.\n"
+" -K <bt_file> | <cert_file>:<key_file[#password]>[:<ca_cert_file>], --k8s-api-cert=<bt_file> | <cert_file>:<key_file[#password]>[:<ca_cert_file>]\n"
+"                    Use the provided files names to authenticate user and (optionally) verify the K8S API\n"
+"                    server identity.\n"
+"                    Each entry must specify full (absolute, or relative to the current directory) path\n"
+"                    to the respective file.\n"
+"                    Private key password is optional (needed only if key is password protected).\n"
+"                    CA certificate is optional. For all files, only PEM file format is supported. \n"
+"                    Specifying CA certificate only is obsoleted - when single entry is provided \n"
+"                    for this option, it will be interpreted as the name of a file containing bearer token.\n"
+"                    Note that the format of this command-line option prohibits use of files whose names contain\n"
+"                    ':' or '#' characters in the file name.\n"
+"                    Option can also be provided via the environment variable SYSDIG_K8S_API_CERT.\n"
 " -l, --list         List all the fields that can be used in views.\n"
 " --logfile=<file>\n"
 "                    Print program logs into the given file.\n"
@@ -121,7 +128,7 @@ static void usage()
 "1. you can either see real time data, or analyze a trace file by using the -r\n"
 "   command line flag.\n"
 "2. you can switch to a different view by using the F2 key.\n"
-"3. You can to drill down into a selection by typing enter.\n"
+"3. You can drill down into a selection by typing enter.\n"
 "   You can navigate back by typing backspace.\n"
 "4. you can observe reads and writes (F5) or see sysdig events (F6) for any\n"
 "   selection.\n"
@@ -233,6 +240,7 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 	bool m_raw_output = false;
 	string* k8s_api = 0;
 	string* k8s_api_cert = 0;
+	string* mesos_api = 0;
 	bool xt1002_available = false;
 
 	static struct option long_options[] =
@@ -243,6 +251,7 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 		{"k8s-api", required_argument, 0, 'k'},
 		{"k8s-api-cert", required_argument, 0, 'K' },
 		{"list", optional_argument, 0, 'l' },
+		{"mesos-api", required_argument, 0, 'm'},
 		{"numevents", required_argument, 0, 'n' },
 		{"print", required_argument, 0, 'p' },
 		{"readfile", required_argument, 0, 'r' },
@@ -269,7 +278,7 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 		// Parse the args
 		//
 		while((op = getopt_long(argc, argv,
-			"d:Ehk:K:lNn:p:r:s:v:", long_options, &long_index)) != -1)
+			"d:Ehk:K:lm:Nn:p:r:s:v:", long_options, &long_index)) != -1)
 		{
 			switch(op)
 			{
@@ -311,6 +320,9 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 			case 'l':
 				list_flds = true;
 				break;
+			case 'm':
+				mesos_api = new string(optarg);
+				break;
 			case 'N':
 				inspector->set_hostname_and_port_resolution_mode(false);
 				break;
@@ -342,6 +354,7 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 			case 'r':
 				infiles.push_back(optarg);
 				k8s_api = new string();
+				mesos_api = new string();
 				break;
 			case 's':
 				snaplen = atoi(optarg);
@@ -385,7 +398,7 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 		//
 		if(list_flds)
 		{
-			list_fields(false);
+			list_fields(false, false);
 
 			res.m_res = EXIT_SUCCESS;
 			goto exit;
@@ -435,7 +448,7 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 			//
 			// Check if xterm-1002 is available
 			//
-			xt1002_available =(tgetent(NULL, MOUSE_CAPABLE_TERM) != 0);
+			xt1002_available = (tgetent(NULL, MOUSE_CAPABLE_TERM) != 0);
 
 			if(xt1002_available)
 			{
@@ -528,8 +541,6 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 			//
 			// Launch the capture
 			//
-			bool open_success = true;
-
 			if(infiles.size() != 0)
 			{
 				//
@@ -548,6 +559,8 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 				// No file to open, this is a live capture
 				//
 #if defined(HAS_CAPTURE)
+				bool open_success = true;
+				
 				try
 				{
 					inspector->open("");
@@ -633,6 +646,24 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 				k8s_api_cert = 0;
 			}
 
+			//
+			// run mesos, if required
+			//
+			if(mesos_api)
+			{
+				inspector->init_mesos_client(mesos_api);
+			}
+			else if(char* mesos_api_env = getenv("SYSDIG_MESOS_API"))
+			{
+				if(mesos_api_env != NULL)
+				{
+					mesos_api = new string(mesos_api_env);
+					inspector->init_mesos_client(mesos_api);
+				}
+			}
+			delete mesos_api;
+			mesos_api = 0;
+
 			//
 			// Start the capture loop
 			//
@@ -649,14 +680,14 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 	catch(sinsp_capture_interrupt_exception&)
 	{
 	}
-	catch(sinsp_exception& e)
+	catch(std::exception& e)
 	{
 		errorstr = e.what();
 		res.m_res = EXIT_FAILURE;
 	}
 	catch(...)
 	{
-		errorstr = "uncatched exception";
+		errorstr = "uncaught exception";
 		res.m_res = EXIT_FAILURE;
 	}
 
