@@ -138,21 +138,34 @@ static void usage()
 #endif
 " -j, --json         Emit output as json, data buffer encoding will depend from the\n"
 "                    print format selected.\n"
-" -k, --k8s-api <url>\n"
+" -k <url>, --k8s-api=<url>\n"
 "                    Enable Kubernetes support by connecting to the API server\n"
 "                    specified as argument. E.g. \"http://admin:password@127.0.0.1:8080\".\n"
 "                    The API server can also be specified via the environment variable\n"
 "                    SYSDIG_K8S_API.\n"
-" -K <file_name>, --k8s-api-cert=<file_name>\n"
-"                    Use the provided certificate file name to authenticate with the K8S API server.\n"
-"                    Filename must be a full absolute or relative (to the current directory) path\n"
-"                    to the certificate file.\n"
-"                    The certificate can also be specified via the environment variable\n"
-"                    SYSDIG_K8S_API_CERT.\n"
+" -K <bt_file> | <cert_file>:<key_file[#password]>[:<ca_cert_file>], --k8s-api-cert=<bt_file> | <cert_file>:<key_file[#password]>[:<ca_cert_file>]\n"
+"                    Use the provided files names to authenticate user and (optionally) verify the K8S API\n"
+"                    server identity.\n"
+"                    Each entry must specify full (absolute, or relative to the current directory) path\n"
+"                    to the respective file.\n"
+"                    Private key password is optional (needed only if key is password protected).\n"
+"                    CA certificate is optional. For all files, only PEM file format is supported. \n"
+"                    Specifying CA certificate only is obsoleted - when single entry is provided \n"
+"                    for this option, it will be interpreted as the name of a file containing bearer token.\n"
+"                    Note that the format of this command-line option prohibits use of files whose names contain\n"
+"                    ':' or '#' characters in the file name.\n"
+"                    Option can also be provided via the environment variable SYSDIG_K8S_API_CERT.\n"
 " -L, --list-events  List the events that the engine supports\n"
 " -l, --list         List the fields that can be used for filtering and output\n"
 "                    formatting. Use -lv to get additional information for each\n"
 "                    field.\n"
+" --list-markdown    like -l, but produces markdown output\n"
+" -m <url[,marathon_url]>, --mesos-api=<url[,marathon_url]>\n"
+"                    Enable Mesos support by connecting to the API server\n"
+"                    specified as argument. E.g. \"http://admin:password@127.0.0.1:5050\".\n"
+"                    Marathon url is optional and defaults to Mesos address, port 8080.\n"
+"                    The API servers can also be specified via the environment variable\n"
+"                    SYSDIG_MESOS_API.\n"
 " -M <num_seconds>   Stop collecting after <num_seconds> reached.\n"
 " -N                 Don't convert port numbers to names.\n"
 " -n <num>, --numevents=<num>\n"
@@ -162,6 +175,7 @@ static void usage()
 "                    Specify the format to be used when printing the events.\n"
 "                    With -pc or -pcontainer will use a container-friendly format.\n"
 "                    With -pk or -pkubernetes will use a kubernetes-friendly format.\n"
+"                    With -pm or -pmesos will use a mesos-friendly format.\n"
 "                    See the examples section below for more info.\n"
 " -q, --quiet        Don't print events on the screen\n"
 "                    Useful when dumping to disk.\n"
@@ -227,6 +241,8 @@ static void usage()
 "%%evt.num %%evt.outputtime %%evt.cpu %%container.name (%%container.id) %%proc.name (%%thread.tid:%%thread.vtid) %%evt.dir %%evt.type %%evt.info\n\n"
 "Using -pk or -pkubernetes, the default format will be changed to a kubernetes-friendly one:\n\n"
 "%%evt.num %%evt.outputtime %%evt.cpu %%k8s.pod.name (%%container.id) %%proc.name (%%thread.tid:%%thread.vtid) %%evt.dir %%evt.type %%evt.info\n\n"
+"Using -pm or -pmesos, the default format will be changed to a mesos-friendly one:\n\n"
+"%%evt.num %%evt.outputtime %%evt.cpu %%mesos.task.name (%%container.id) %%proc.name (%%thread.tid:%%thread.vtid) %%evt.dir %%evt.type %%evt.info\n\n"
 "Examples:\n\n"
 " Capture all the events from the live system and print them to screen\n"
 "   $ sysdig\n\n"
@@ -379,7 +395,9 @@ static void parse_chisel_args(sinsp_chisel* ch, sinsp* inspector, int optind, in
 					{
 						try
 						{
-							sinsp_filter df(inspector, testflt);
+							sinsp_filter_compiler compiler(inspector, testflt);
+							sinsp_filter* s = compiler.compile();
+							delete s;
 						}
 						catch(...)
 						{
@@ -498,7 +516,7 @@ captureinfo do_inspect(sinsp* inspector,
 	sinsp_evt* ev;
 	string line;
 	double last_printed_progress_pct = 0;
-        int duration_start = 0;
+	int duration_start = 0;
 
 	if(json)
 	{
@@ -529,7 +547,6 @@ captureinfo do_inspect(sinsp* inspector,
 			handle_end_of_file(print_progress, formatter);
 			break;
 		}
-
 		res = inspector->next(&ev);
 
 		if(res == SCAP_TIMEOUT)
@@ -680,6 +697,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 	bool is_filter_display = false;
 	bool verbose = false;
 	bool list_flds = false;
+	bool list_flds_markdown = false;
 	bool print_progress = false;
 	bool compress = false;
 	sinsp_evt::param_fmt event_buffer_format = sinsp_evt::PF_NORMAL;
@@ -699,6 +717,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 	vector<summary_table_entry>* summary_table = NULL;
 	string* k8s_api = 0;
 	string* k8s_api_cert = 0;
+	string* mesos_api = 0;
 
 	// These variables are for the cycle_writer engine
 	int duration_seconds = 0;
@@ -731,6 +750,8 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 		{"k8s-api-cert", required_argument, 0, 'K' },
 		{"list", no_argument, 0, 'l' },
 		{"list-events", no_argument, 0, 'L' },
+		{"list-markdown", no_argument, 0, 0 },
+		{"mesos-api", required_argument, 0, 'm'},
 		{"numevents", required_argument, 0, 'n' },
 		{"progress", required_argument, 0, 'P' },
 		{"print", required_argument, 0, 'p' },
@@ -769,7 +790,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
                                         "C:"
                                         "dDEe:F"
                                         "G:"
-                                        "hi:jk:K:lLM:Nn:Pp:qr:Ss:t:v"
+                                        "hi:jk:K:lLm:M:Nn:Pp:qr:Ss:t:v"
                                         "W:"
                                         "w:xXz", long_options, &long_index)) != -1)
 		{
@@ -928,6 +949,9 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 				list_events(inspector);
 				delete inspector;
 				return sysdig_init_res(EXIT_SUCCESS);
+			case 'm':
+				mesos_api = new string(optarg);
+				break;
 			case 'M':
 				duration_to_tot = atoi(optarg);
 				if(duration_to_tot <= 0)
@@ -963,9 +987,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			case 'p':
 				if(string(optarg) == "p")
 				{
-					//
 					// -pp shows the default output format, useful if the user wants to tweak it.
-					//
 					printf("%s\n", output_format.c_str());
 					delete inspector;
 					return sysdig_init_res(EXIT_SUCCESS);
@@ -974,9 +996,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 				{
 					output_format = "*%evt.num %evt.outputtime %evt.cpu %container.name (%container.id) %proc.name (%thread.tid:%thread.vtid) %evt.dir %evt.type %evt.info";
 
-					//
 					// This enables chisels to determine if they should print container information
-					//
 					if(inspector != NULL)
 					{
 						inspector->set_print_container_data(true);
@@ -986,9 +1006,17 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 				{
 					output_format = "*%evt.num %evt.outputtime %evt.cpu %k8s.pod.name (%container.id) %proc.name (%thread.tid:%thread.vtid) %evt.dir %evt.type %evt.info";
 
-					//
 					// This enables chisels to determine if they should print container information
-					//
+					if(inspector != NULL)
+					{
+						inspector->set_print_container_data(true);
+					}
+				}
+				else if(string(optarg) == "m" || string(optarg) == "mesos")
+				{
+					output_format = "*%evt.num %evt.outputtime %evt.cpu %mesos.task.name (%container.id) %proc.name (%thread.tid:%thread.vtid) %evt.dir %evt.type %evt.info";
+
+					// This enables chisels to determine if they should print container information
 					if(inspector != NULL)
 					{
 						inspector->set_print_container_data(true);
@@ -1006,6 +1034,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			case 'r':
 				infiles.push_back(optarg);
 				k8s_api = new string();
+				mesos_api = new string();
 				break;
 			case 'S':
 				summary_table = new vector<summary_table_entry>;
@@ -1107,6 +1136,12 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			{
 				filter_proclist_flag = true;
 			}
+
+			if(string(long_options[long_index].name) == "list-markdown")
+			{
+				list_flds = true;
+				list_flds_markdown = true;
+			}
 		}
 
 		//
@@ -1150,11 +1185,11 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 				// -ll shows the fields verbosely, i.e. with more information
 				// like the type
 				//
-				list_fields(true);
+				list_fields(true, list_flds_markdown);
 			}
 			else
 			{
-				list_fields(false);
+				list_fields(false, list_flds_markdown);
 			}
 
 			res.m_res = EXIT_SUCCESS;
@@ -1180,7 +1215,8 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 
 			if(is_filter_display)
 			{
-				display_filter = new sinsp_filter(inspector, filter);
+				sinsp_filter_compiler compiler(inspector, filter);
+				display_filter = compiler.compile();
 			}
 #else
 			fprintf(stderr, "filtering not compiled.\n");
@@ -1252,8 +1288,6 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			//
 			// Launch the capture
 			//
-			bool open_success = true;
-
 			if(infiles.size() != 0)
 			{
 				initialize_chisels();
@@ -1276,6 +1310,8 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 				// No file to open, this is a live capture
 				//
 #if defined(HAS_CAPTURE)
+				bool open_success = true;
+				
 				if(print_progress)
 				{
 					fprintf(stderr, "the -P flag cannot be used with live captures.\n");
@@ -1302,7 +1338,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 
 					if(system("modprobe " PROBE_NAME " > /dev/null 2> /dev/null"))
 					{
-						fprintf(stderr, "Unable to load the driver\n");						
+						fprintf(stderr, "Unable to load the driver\n");
 					}
 
 					inspector->open("");
@@ -1351,7 +1387,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 						k8s_api_cert = new string(k8s_cert_env);
 					}
 				}
-				inspector->init_k8s_client(k8s_api, k8s_api_cert);
+				inspector->init_k8s_client(k8s_api, k8s_api_cert, verbose);
 				k8s_api = 0;
 				k8s_api_cert = 0;
 			}
@@ -1367,7 +1403,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 						}
 					}
 					k8s_api = new string(k8s_api_env);
-					inspector->init_k8s_client(k8s_api, k8s_api_cert);
+					inspector->init_k8s_client(k8s_api, k8s_api_cert, verbose);
 				}
 				else
 				{
@@ -1378,9 +1414,27 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 				k8s_api_cert = 0;
 			}
 
+			//
+			// run mesos, if required
+			//
+			if(mesos_api)
+			{
+				inspector->init_mesos_client(mesos_api, verbose);
+			}
+			else if(char* mesos_api_env = getenv("SYSDIG_MESOS_API"))
+			{
+				if(mesos_api_env != NULL)
+				{
+					mesos_api = new string(mesos_api_env);
+					inspector->init_mesos_client(mesos_api, verbose);
+				}
+			}
+			delete mesos_api;
+			mesos_api = 0;
+
 			cinfo = do_inspect(inspector,
 				cnt,
-                                duration_to_tot,
+				duration_to_tot,
 				quiet,
 				jflag,
 				unbuf_flag,
